package com.ruoyi.web.controller.qdtl;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.core.text.Convert;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.qdtl.domain.TlArea;
import com.ruoyi.qdtl.service.ITlAreaService;
import com.ruoyi.qdtl.service.ITlInspectionDeviceService;
import com.ruoyi.qdtl.service.ITlInspectionLocationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;

/**
 * 区域管理Controller
 *
 * @author ruoyi
 * @date 2022-02-21
 */
@RestController
@RequestMapping("/qdtl/area")
public class TlAreaController extends BaseController {
    @Autowired
    private ITlAreaService tlAreaService;
    @Autowired
    private ITlInspectionDeviceService tlInspectionDeviceService;
    @Autowired
    private ITlInspectionLocationService tlInspectionLocationService;

    /**
     * 查询区域管理列表
     */
    @GetMapping("/list")
    public TableDataInfo list(TlArea tlArea) {
        startPage();
        List<TlArea> list = tlAreaService.selectTlAreaList(tlArea);
        if (list.size() > 0) {
            // 查询设备数和巡检点数量
            Map<String, Integer> deviceArea = tlInspectionDeviceService.queryDeviceByArea();
            Map<String, Integer> locationArea = tlInspectionLocationService.queryLocationByArea();
            for (TlArea area : list) {
                if ("1".equals(area.getAreaType())) {
                    area.setDeviceCount(Convert.toInt(deviceArea.get(Convert.toStr(area.getId())), 0));
                    area.setLocationCount(Convert.toInt(locationArea.get(Convert.toStr(area.getId())), 0));
                }
            }
        }
        return getDataTable(list);
    }

    /**
     * 导出区域管理列表
     */
    @Log(title = "区域管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TlArea tlArea) {
        List<TlArea> list = tlAreaService.selectTlAreaList(tlArea);
        ExcelUtil<TlArea> util = new ExcelUtil<TlArea>(TlArea.class);
        util.exportExcel(response, list, "区域管理数据");
    }

    /**
     * 获取区域管理详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(tlAreaService.selectTlAreaById(id));
    }

    /**
     * 新增区域管理
     */
    @Log(title = "区域管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TlArea tlArea) {
        return toAjax(tlAreaService.insertTlArea(tlArea));
    }

    /**
     * 修改区域管理
     */
    @Log(title = "区域管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TlArea tlArea) {
        return toAjax(tlAreaService.updateTlArea(tlArea));
    }

    /**
     * 删除区域管理
     */
    @Log(title = "区域管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(tlAreaService.deleteTlAreaByIds(ids));
    }
}
