package com.ruoyi.web.controller.qdtl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.qdtl.domain.IncidentAreaNum;
import com.ruoyi.qdtl.domain.IncidentDetailAreaNum;
import com.ruoyi.qdtl.domain.MonitorStatic;
import com.ruoyi.qdtl.domain.TlArea;
import com.ruoyi.qdtl.domain.TlIncident;
import com.ruoyi.qdtl.domain.TlInspectionLocation;
import com.ruoyi.qdtl.domain.TlInspectionLocationLog;
import com.ruoyi.qdtl.domain.UserCheckPoint;
import com.ruoyi.qdtl.service.IQdCommonService;
import com.ruoyi.qdtl.service.ITlAreaService;
import com.ruoyi.qdtl.service.ITlIncidentService;
import com.ruoyi.qdtl.service.ITlInspectionLocationLogService;
import com.ruoyi.qdtl.service.ITlInspectionLocationService;
import com.ruoyi.qdtl.service.ITlMonitorService;
import com.ruoyi.qdtl.service.ITlPlanRecordService;
import com.ruoyi.qdtl.service.ITlSinglePawnService;
import com.ruoyi.system.domain.SysConfig;
import com.ruoyi.system.service.ISysConfigService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import sun.misc.BASE64Decoder;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 监控设备管理Controller
 *
 * @author ruoyi
 * @date 2022-03-02
 */
@RestController
@RequestMapping("/qdtl/common")
@Slf4j
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class TlCommonController extends BaseController {

    private final RedisCache redisCache;
    private final ISysConfigService sysConfigService;
    private final ITlAreaService tlAreaService;
    private final ITlInspectionLocationService tlInspectionLocationService;
    private final ITlInspectionLocationLogService tlInspectionLocationLogService;
    private final ITlMonitorService tlMonitorService;
    private final ITlPlanRecordService planRecordService;
    private final IQdCommonService qdCommonService;

    private final ITlSinglePawnService singlePawnService;
    private final ITlIncidentService incidentService;


    /**
     * 查询监控设备管理列表
     */
    @PostMapping("/http")
    public AjaxResult list(@RequestBody JSONObject jsonObject) {
        String url = jsonObject.getStr("url");
        String method = jsonObject.getStr("method");
        if (StrUtil.isBlank(method)) {
            method = "post";
        }
        String params = jsonObject.getStr("params");
        // 通过params解析缓存key
        String paramsPath = JSONUtil.parseObj(params).getStr("path");
        String result = "";
        if ("post".equals(method)) {
            try {
                result = HttpRequest.post(url).timeout(8000).body(params).execute().body();
                // 结果集存入缓存
                redisCache.setCacheObject("tl:http:" + paramsPath, result);
            } catch (Exception e) {
                log.error("http请求失败：{}", e.getMessage());
                // 从缓存中取值
                result = redisCache.getCacheObject("tl:http:" + paramsPath);
            }

        } else if ("get".equals(method)) {
            result = HttpUtil.get(url);
        }

        return AjaxResult.success("请求成功", result);
    }

    @GetMapping("/railwayLine")
    public AjaxResult railwayLine(@RequestParam(required = false) String key) {
        List<SysConfig> list = sysConfigService.selectConfigByLeftKey("tl.railway.");
        return AjaxResult.success(list);
    }

    /**
     * 地图上各类资源数据
     *
     * @return
     */
    @GetMapping("/resource")
    public AjaxResult resource(@RequestParam(required = false) String date) {
        Map<String, Object> map = new HashMap<>(8);
        // 区域
        List<TlArea> areas = tlAreaService.selectAreaList();
        map.put("area", areas);
        // 静态资源
        // 巡检点
        List<TlInspectionLocation> staticLocation = tlInspectionLocationService.selectLocationList(null);
        map.put("staticLocation", staticLocation);

        // 动态资源
        // 巡检轨迹
        if (StrUtil.isBlank(date)) {
            date = DateUtil.today();
        }
        List<TlInspectionLocationLog> deviceTrails = tlInspectionLocationLogService.selectListByDate(date);
        // 对轨迹按照设备编码分组
        Map<String, List<TlInspectionLocationLog>> deviceTrail = deviceTrails.stream().collect(Collectors.groupingBy(TlInspectionLocationLog::getDeviceCode));
        map.put("deviceTrail", deviceTrail);

        // 铁路沿线
        List<SysConfig> railway = sysConfigService.selectConfigByLeftKey("tl.railway.");
        map.put("railway", railway);
        // 无人机相关信息
        String drone = sysConfigService.selectConfigByKey("tl.drone");
        map.put("drone", JSON.parseArray(drone));
        return AjaxResult.success(map);
    }

    /**
     * 治理态势统计
     *
     * @param date
     * @return
     */
    @GetMapping("/govern")
    public AjaxResult govern(@RequestParam(required = false) String date) {
        Map<String, Object> map = new HashMap<>(8);
        if (StrUtil.isBlank(date)) {
            date = DateUtil.today();
        }
        // 巡检里程
        double xunTotalDistance = tlInspectionLocationLogService.queryDistance(date, "");
        map.put("xunTotalDistance", xunTotalDistance);
        // 巡检打卡数
        List<TlInspectionLocationLog> locationLogList = tlInspectionLocationLogService.queryByCondition(date, "", "");
        map.put("checkPointLogNum", locationLogList.size());
        // 视频资源统计
        List<MonitorStatic> cameraslist = tlMonitorService.queryMonitorStatic();
        map.put("cameraslist", cameraslist);
        // 巡检考核
        List<UserCheckPoint> checkStatic = tlInspectionLocationLogService.queryUserGroup();
        map.put("checkStatic", checkStatic);
        // 当日巡检完成率(总以及各个镇)
        Map<String, Object> checkPercent = planRecordService.queryCheckPercent(date);
        map.put("areaStatics", checkPercent.get("areaStatics"));
        map.put("totalStatic", checkPercent.get("totalStatic"));
        // 周巡检里程统计
        DateTime endDate = DateUtil.yesterday();
        DateTime startDate = DateUtil.offsetDay(endDate, -6);
        List<JSONObject> locationLogDaysList = tlInspectionLocationLogService.queryDistanceByDateRange(DateUtil.formatDate(startDate), DateUtil.formatDate(endDate));
        map.put("locationLogDaysList", locationLogDaysList);
        // 隐患分析
        List<TlIncident> tlIncidents = incidentService.selectTlIncidentList(new TlIncident());
        long unDealTotal = tlIncidents.stream().filter(obj -> "1".equals(obj.getStatus())).count();
        Map<String, Map<String, Integer>> incidentDetailCount = tlIncidents.stream().collect(Collectors.groupingBy(TlIncident::getArea, Collectors.groupingBy(TlIncident::getStatus, Collectors.summingInt(e -> 1))));
        Map<String, Map<String, Integer>> modifiedMap = incidentDetailCount.entrySet().stream()
                .collect(Collectors.toMap(
                        Map.Entry::getKey,
                        entry -> entry.getValue().entrySet().stream()
                                .collect(Collectors.toMap(
                                        e -> {
                                            if ("1".equals(e.getKey())) {
                                                return "key1";
                                            } else if ("2".equals(e.getKey())) {
                                                return "key2";
                                            }
                                            return e.getKey();
                                        },
                                        Map.Entry::getValue
                                ))
                ));
        Map<String, Integer> incidentCount = tlIncidents.stream().collect(Collectors.groupingBy(TlIncident::getArea, Collectors.summingInt(e -> 1)));
        map.put("unDealTotal", unDealTotal); // 未处理隐患总数
        map.put("incidentTotal", tlIncidents.size()); // 隐患总数
        TlArea tlArea = new TlArea();
        tlArea.setAreaType("1");
        List<TlArea> areas = tlAreaService.selectTlAreaList(tlArea);
        List<IncidentAreaNum> areaIncidentTotals = new ArrayList<>();
        List<IncidentDetailAreaNum> incidentDetailCounts = new ArrayList<>();
        for (TlArea area : areas) {
            IncidentAreaNum incidentAreaNum = new IncidentAreaNum();
            incidentAreaNum.setAreaName(area.getAreaName());
            incidentAreaNum.setNum(incidentCount.getOrDefault(Convert.toStr(area.getId()), 0));
            areaIncidentTotals.add(incidentAreaNum);

            IncidentDetailAreaNum incidentDetailAreaNum = new IncidentDetailAreaNum();
            incidentDetailAreaNum.setAreaName(area.getAreaName());
            incidentDetailAreaNum.setDetail(modifiedMap.get(Convert.toStr(area.getId())));
            incidentDetailCounts.add(incidentDetailAreaNum);
        }
        map.put("areaIncidentTotal", areaIncidentTotals); // 各区域隐患占比
        map.put("areaIncidentDetail", incidentDetailCounts); // 各区域隐患分项占比

        return AjaxResult.success(map);
    }

    @Value(value = "${hls.path}")
    private String path;
    @Value(value = "${hls.url}")
    private String url;
    @Value(value = "${hls.ffmpegpath}")
    private String ffmpegpath;
    private Map<String, StringUtils.PrintStream> rtmptohlsmap = new HashMap<>();

    @GetMapping("/rtmp/{from}")
    public AjaxResult getHlsCode(@PathVariable("from") String from) {
        Object data = "";
        try {
            from = new String(new BASE64Decoder().decodeBuffer(from), "UTF-8");
            String p[] = from.split("/");
            if (rtmptohlsmap.get(from) != null && rtmptohlsmap.get(from).isAlive()) {

            } else {
                File pc = new File(path + p[p.length - 1] + "");
                if (!pc.exists()) {
                    pc.mkdir();
                    pc.setWritable(true, false);
                    File pc1 = new File(path + p[p.length - 1] + "/playlist.m3u8");
                    pc1.createNewFile();
                    pc1.setWritable(true, false);

                }
                rtmptohlsmap.put(from, StringUtils.push(ffmpegpath, from, path + p[p.length - 1] + "/playlist.m3u8"));
            }
            data = url + p[p.length - 1] + "/playlist.m3u8";
        } catch (Exception e) {

        }
        return AjaxResult.success(data);
    }

    @GetMapping("/monthStatic")
    public AjaxResult monthStatic(@RequestParam(required = false) Long areaId, @RequestParam String startDate, @RequestParam String endDate) {
        return AjaxResult.success(qdCommonService.monthStatic(areaId, startDate, endDate));
    }


    //获取单兵播放列表
    @GetMapping("/singlepawnplays")
    public AjaxResult singlepawnplays() {
        return AjaxResult.success(singlePawnService.getSinglePawnList());
    }

}
