package com.ruoyi.web.controller.qdtl;

import cn.afterturn.easypoi.word.WordExportUtil;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.qdtl.domain.AreaCheckInfoStatic;
import com.ruoyi.qdtl.domain.DocumentExample;
import com.ruoyi.qdtl.domain.TlDocument;
import com.ruoyi.qdtl.service.ITlDocumentService;
import com.ruoyi.qdtl.service.ITlPlanRecordService;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 公文管理Controller
 *
 * @author ruoyi
 * @date 2022-05-06
 */
@RestController
@RequestMapping("/qdtl/document")
public class TlDocumentController extends BaseController {
    @Autowired
    private ITlDocumentService tlDocumentService;
    @Autowired
    private ITlPlanRecordService planRecordService;

    /**
     * 查询公文管理列表
     */
    @GetMapping("/list")
    public TableDataInfo list(TlDocument tlDocument) {
        startPage();
        List<TlDocument> list = tlDocumentService.selectTlDocumentList(tlDocument);
        return getDataTable(list);
    }

    /**
     * 根据日期区间查询巡检打卡情况统计
     *
     * @param startDate
     * @param endDate
     * @return
     */
    @GetMapping("/queryCheck")
    public AjaxResult queryCheck(@RequestParam String startDate, @RequestParam String endDate) {
        return AjaxResult.success(planRecordService.queryCheckInfo(startDate, endDate));
    }

    /**
     * 导出公文管理列表
     */
    @Log(title = "公文管理", businessType = BusinessType.EXPORT)
    @RequestMapping(value = "/export", method = {RequestMethod.GET, RequestMethod.POST})
    public void export(@RequestParam Long id, HttpServletResponse response) {
        // 查询公文明细
        TlDocument tlDocument = tlDocumentService.selectTlDocumentById(id);
        // 处理发文日期
        tlDocument.setPublishDate(DateUtil.format(DateUtil.parseDate(tlDocument.getPublishDate()), DatePattern.CHINESE_DATE_PATTERN));
        // 处理典型案例
        Object basicExample = tlDocument.getBasicExample();
        if (ObjectUtil.isNotEmpty(basicExample)) {
            List<DocumentExample> documentExamples = JSONUtil.toList(basicExample.toString(), DocumentExample.class);
            tlDocument.setBasicExample(documentExamples.stream().map(DocumentExample::getContent).collect(Collectors.joining("\r\t")));
        }
        // 处理存在问题
        Object question = tlDocument.getQuestion();
        if (ObjectUtil.isNotEmpty(question)) {
            List<DocumentExample> documentQuestion = JSONUtil.toList(question.toString(), DocumentExample.class);
            tlDocument.setQuestion(documentQuestion.stream().map(DocumentExample::getContent).collect(Collectors.joining("\r\t")));
        }
        // 处理工作建议
        Object advice = tlDocument.getAdvice();
        if (ObjectUtil.isNotEmpty(advice)) {
            List<DocumentExample> documentAdvice = JSONUtil.toList(advice.toString(), DocumentExample.class);
            tlDocument.setAdvice(documentAdvice.stream().map(DocumentExample::getContent).collect(Collectors.joining("\r\t")));
        }

        Map<String, Object> map = BeanUtil.beanToMap(tlDocument);
        // 查询巡检的统计数据
        String infoTitle = StrUtil.concat(true, DateUtil.format(DateUtil.parseDate(tlDocument.getStartStatistic()), DatePattern.CHINESE_DATE_PATTERN),
                "--",
                DateUtil.format(DateUtil.parseDate(tlDocument.getEndStatistic()), DatePattern.CHINESE_DATE_PATTERN),
                "铁路沿线巡防员打卡情况汇总");
        map.put("infoTitle", infoTitle);
        List<AreaCheckInfoStatic> list = planRecordService.queryCheckInfo(tlDocument.getStartStatistic(), tlDocument.getEndStatistic());
        int i = 1;
        for (AreaCheckInfoStatic areaCheckInfoStatic : list) {
            areaCheckInfoStatic.setIndex(i);
            i++;
        }
        map.put("checkList", list);

        String fileName = "document_" + DateUtil.format(new Date(), "yyyyMMddHHmmss") + ".docx";
        try {
            XWPFDocument doc = WordExportUtil.exportWord07(
                    "excel/document_template.docx", map);
            FileOutputStream fos = new FileOutputStream(ExcelUtil.getAbsoluteFile(fileName));
            doc.write(fos);
            // 设置强制下载不打开
            response.setContentType("application/force-download");
            // 设置文件名
            response.addHeader("Content-Disposition", "attachment;fileName=" + fileName);
            OutputStream out = response.getOutputStream();
            doc.write(out);
            out.close();
            /*for (String s : tlDocument.getFileUrl().split(",")) {
                BufferedInputStream br = FileUtil.getInputStream(s);
                byte[] buf = new byte[1024];
                int len = 0;
                OutputStream out2 = response.getOutputStream();
                while ((len = br.read(buf)) > 0)
                    out2.write(buf, 0, len);
                br.close();
                out2.close();
            }*/
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            delFileWord(ExcelUtil.getAbsoluteFile(fileName));
        }
    }

    /**
     * 获取公文管理详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(tlDocumentService.selectTlDocumentById(id));
    }

    /**
     * 新增公文管理
     */
    @Log(title = "公文管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TlDocument tlDocument) {
        tlDocument.setBasicExample(JSONUtil.toJsonStr(tlDocument.getBasicExample()));
        tlDocument.setQuestion(JSONUtil.toJsonStr(tlDocument.getQuestion()));
        tlDocument.setAdvice(JSONUtil.toJsonStr(tlDocument.getAdvice()));
        return AjaxResult.success(tlDocumentService.insertTlDocument(tlDocument));
    }

    /**
     * 修改公文管理
     */
    @Log(title = "公文管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TlDocument tlDocument) {
        tlDocument.setBasicExample(JSONUtil.toJsonStr(tlDocument.getBasicExample()));
        tlDocument.setQuestion(JSONUtil.toJsonStr(tlDocument.getQuestion()));
        tlDocument.setAdvice(JSONUtil.toJsonStr(tlDocument.getAdvice()));
        return toAjax(tlDocumentService.updateTlDocument(tlDocument));
    }

    /**
     * 删除公文管理
     */
    @Log(title = "公文管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(tlDocumentService.deleteTlDocumentByIds(ids));
    }

    /**
     * 删除零时生成的文件
     */
    public static void delFileWord(String filePath) {
        File file = new File(filePath);
        file.delete();
    }


}
