package com.ruoyi.web.controller.qdtl;

import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.qdtl.domain.TlElectricDevice;
import com.ruoyi.qdtl.service.ITlElectricDeviceService;
import com.ruoyi.system.service.ISysRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * 电动车定位设备Controller
 *
 * @author ruoyi
 * @date 2022-06-03
 */
@RestController
@RequestMapping("/qdtl/electricDevice")
public class TlElectricDeviceController extends BaseController {
    @Autowired
    private ITlElectricDeviceService tlElectricDeviceService;
    @Autowired
    private ISysRoleService sysRoleService;

    /**
     * 查询电动车定位设备列表
     */
    @GetMapping("/list")
    public TableDataInfo list(TlElectricDevice tlElectricDevice) {
        startPage();
        List<TlElectricDevice> list = tlElectricDeviceService.selectTlElectricDeviceList(tlElectricDevice);
        return getDataTable(list);
    }

    /**
     * 同步第三方的设备数据到本地
     *
     * @return
     */
    @PostMapping("/syncDevice")
    public AjaxResult syncDevice() {
        return AjaxResult.success(tlElectricDeviceService.syncDevice());
    }

    @GetMapping("/queryAllList")
    public AjaxResult queryAllList() {
        Set<String> roleSet = sysRoleService.selectRolePermissionByUserId(getUserId());
        List<TlElectricDevice> list = tlElectricDeviceService.queryAllEnableList();
        if (!roleSet.contains("admin")) {
            List<TlElectricDevice> newList = new ArrayList<>();
            // 筛选
            for (TlElectricDevice tlElectricDevice : list) {
                if (roleSet.contains(tlElectricDevice.getAreaId())) {
                    newList.add(tlElectricDevice);
                }
            }
            return AjaxResult.success(newList);
        }
        return AjaxResult.success(list);
    }

    /**
     * 巡逻车统计（根据区域分类，查询总数和在线数）
     *
     * @return
     */
    @GetMapping("/queryCarStaticByArea")
    public AjaxResult queryCarStaticByArea() {
        return AjaxResult.success(tlElectricDeviceService.queryCarStaticByArea());
    }

    /**
     * 导出电动车定位设备列表
     */
    /*@PreAuthorize("@ss.hasPermi('qdtl:device:export')")
    @Log(title = "电动车定位设备", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TlElectricDevice tlElectricDevice) {
        List<TlElectricDevice> list = tlElectricDeviceService.selectTlElectricDeviceList(tlElectricDevice);
        ExcelUtil<TlElectricDevice> util = new ExcelUtil<TlElectricDevice>(TlElectricDevice.class);
        util.exportExcel(response, list, "电动车定位设备数据");
    }*/

    /**
     * 获取电动车定位设备详细信息
     */
    /*@PreAuthorize("@ss.hasPermi('qdtl:device:query')")
    @GetMapping(value = "/{imei}")
    public AjaxResult getInfo(@PathVariable("imei") String imei) {
        return AjaxResult.success(tlElectricDeviceService.selectTlElectricDeviceByImei(imei));
    }*/

    /**
     * 新增电动车定位设备
     *//*
    @PreAuthorize("@ss.hasPermi('qdtl:device:add')")
    @Log(title = "电动车定位设备", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TlElectricDevice tlElectricDevice) {
        return toAjax(tlElectricDeviceService.insertTlElectricDevice(tlElectricDevice));
    }

    *//**
     * 修改电动车定位设备
     *//*
    @PreAuthorize("@ss.hasPermi('qdtl:device:edit')")
    @Log(title = "电动车定位设备", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TlElectricDevice tlElectricDevice) {
        return toAjax(tlElectricDeviceService.updateTlElectricDevice(tlElectricDevice));
    }

    *//**
     * 删除电动车定位设备
     *//*
    @PreAuthorize("@ss.hasPermi('qdtl:device:remove')")
    @Log(title = "电动车定位设备", businessType = BusinessType.DELETE)
    @DeleteMapping("/{imeis}")
    public AjaxResult remove(@PathVariable String[] imeis) {
        return toAjax(tlElectricDeviceService.deleteTlElectricDeviceByImeis(imeis));
    }*/
}
