package com.ruoyi.web.controller.qdtl;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.qdtl.domain.ElectricDeviceLocationRecord;
import com.ruoyi.qdtl.domain.TlElectricDeviceLocation;
import com.ruoyi.qdtl.service.ITlElectricDeviceLocationService;
import com.ruoyi.system.service.ISysDictDataService;
import com.ruoyi.system.service.ISysRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * 电动车定位设备轨迹Controller
 *
 * @author ruoyi
 * @date 2022-06-03
 */
@RestController
@RequestMapping("/qdtl/electricLocation")
public class TlElectricDeviceLocationController extends BaseController {
    @Autowired
    private ITlElectricDeviceLocationService tlElectricDeviceLocationService;
    @Autowired
    private ISysRoleService sysRoleService;
    @Autowired
    private ISysDictDataService dictDataService;

    /**
     * 查询电动车定位设备轨迹列表
     */
    @GetMapping("/list")
    public TableDataInfo list(TlElectricDeviceLocation tlElectricDeviceLocation) {
        startPage();
        List<TlElectricDeviceLocation> list = tlElectricDeviceLocationService.selectTlElectricDeviceLocationList(tlElectricDeviceLocation);
        return getDataTable(list);
    }

    @GetMapping("/queryLocationRecord")
    public AjaxResult queryLocationRecord(@RequestParam String startDate,
                                          @RequestParam String endDate,
                                          @RequestParam(required = false) String areaName,
                                          @RequestParam(required = false) String imei) {
        startDate = startDate + " 00:00:00";
        endDate = endDate + " 23:59:59";
        List<ElectricDeviceLocationRecord> list = tlElectricDeviceLocationService.queryLocationRecord(startDate, endDate, areaName, imei);
        Set<String> roleSet = sysRoleService.selectRolePermissionByUserId(getUserId());
        if (!roleSet.contains("admin")) {
            Set<String> areaSet = new HashSet<>();
            for (String s : roleSet) {
                String tl_xun_area = dictDataService.selectDictLabel("tl_xun_area", s);
                areaSet.add(tl_xun_area);
            }
            if (areaSet.size() > 0) {
                List<ElectricDeviceLocationRecord> newList = new ArrayList<>();
                for (ElectricDeviceLocationRecord electricDeviceLocationRecord : list) {
                    if (areaSet.contains(electricDeviceLocationRecord.getDeviceName().substring(0, 3))) {
                        newList.add(electricDeviceLocationRecord);
                    }
                }
                return AjaxResult.success(newList);
            }
        }
        return AjaxResult.success(list);
    }

    @PostMapping("/syncDeviceLocation")
    public AjaxResult syncDeviceLocation() {
        tlElectricDeviceLocationService.syncDeviceLocation();
        return AjaxResult.success();
    }

    @GetMapping("/getAllElectricDetail")
    public AjaxResult getAllElectricDetail() {
        return AjaxResult.success(tlElectricDeviceLocationService.getAllElectricDetail());
    }

    /**
     * 获取单个设备轨迹
     *
     * @return
     */
    @GetMapping("/getSingleElectricLocation")
    public AjaxResult getSingleElectricLocation(@RequestParam String startDate,
                                                @RequestParam String endDate,
                                                @RequestParam String imei) {
        startDate = startDate + " 00:00:00";
        endDate = endDate + " 23:59:59";
        return AjaxResult.success(tlElectricDeviceLocationService.getSingleElectricLocation(startDate, endDate, imei));
    }

    /**
     * 导出电动车定位设备轨迹列表
     */
    @Log(title = "电动车定位设备轨迹", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response,
                       String startDate,
                       String endDate,
                       String areaName,
                       String imei) {
        startDate = startDate + " 00:00:00";
        endDate = endDate + " 23:59:59";
        List<ElectricDeviceLocationRecord> list = tlElectricDeviceLocationService.queryLocationRecord(startDate, endDate, areaName, imei);
        ExcelUtil<ElectricDeviceLocationRecord> util = new ExcelUtil<>(ElectricDeviceLocationRecord.class);
        util.exportExcel(response, list, "电动车定位设备轨迹数据");
    }

}
