package com.ruoyi.web.controller.qdtl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysDictData;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.file.ImageUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.qdtl.domain.TlIncident;
import com.ruoyi.qdtl.service.ITlIncidentService;
import com.ruoyi.system.service.ISysDictDataService;
import com.ruoyi.system.service.ISysRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 巡检事件管理Controller
 *
 * @author ruoyi
 * @date 2022-08-09
 */
@RestController
@RequestMapping("/qdtl/incident")
public class TlIncidentController extends BaseController {
    @Autowired
    private ITlIncidentService tlIncidentService;
    @Autowired
    private ISysRoleService sysRoleService;

    @Autowired
    private ISysDictDataService sysDictDataService;

    /**
     * 查询巡检事件管理列表
     */
    @GetMapping("/list")
    public TableDataInfo list(TlIncident tlIncident) {
        // 根据当前用户角色，如果是巡检员的话，则只查询自己的
        // 如果是巡检员管理员的话，查询接收人是自己的
        Set<String> roleSet = sysRoleService.selectRolePermissionByUserId(getUserId());
        if (roleSet.contains("xun")) {
            tlIncident.setCreateBy(getUsername());
        } else if (roleSet.contains("xun_manage")) {
            tlIncident.setAcceptUser(getUsername());
        }
        if (StrUtil.isBlank(tlIncident.getQueryFlag())) {
            tlIncident.setStatus("1");
        }
        startPage();
        List<TlIncident> list = tlIncidentService.selectTlIncidentList(tlIncident);
        return getDataTable(list);
    }

    @GetMapping("/count")
    public AjaxResult count() {
        TlIncident tlIncident = new TlIncident();
        // 根据当前用户角色，如果是巡检员的话，则只查询自己的
        // 如果是巡检员管理员的话，查询接收人是自己的
        Set<String> roleSet = sysRoleService.selectRolePermissionByUserId(getUserId());
        if (roleSet.contains("xun")) {
            tlIncident.setCreateBy(getUsername());
        } else if (roleSet.contains("xun_manage")) {
            tlIncident.setAcceptUser(getUsername());
        }
        Map<String, Object> result = new HashMap<>();
        List<TlIncident> list = tlIncidentService.selectTlIncidentList(tlIncident);
        long unDeal = list.stream().filter(a -> StrUtil.equals(a.getStatus(), "1")).count();
        long deal = list.stream().filter(a -> StrUtil.equals(a.getStatus(), "2")).count();
        result.put("unDeal", unDeal);
        result.put("deal", deal);
        return AjaxResult.success(result);
    }

    /**
     * 导出巡检事件管理列表
     */
    @Log(title = "巡检事件管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TlIncident tlIncident) {
        // 根据当前用户角色，如果是巡检员的话，则只查询自己的
        // 如果是巡检员管理员的话，查询接收人是自己的
        Set<String> roleSet = sysRoleService.selectRolePermissionByUserId(getUserId());
        if (roleSet.contains("xun")) {
            tlIncident.setCreateBy(getUsername());
        } else if (roleSet.contains("xun_manage")) {
            tlIncident.setAcceptUser(getUsername());
        }
        if (StrUtil.isBlank(tlIncident.getBeginTime())) {
            tlIncident.setBeginTime(DateUtil.thisYear() + "-" + (DateUtil.thisMonth() + 1) + "-01 00:00:00");
        }
        if (StrUtil.isBlank(tlIncident.getEndTime())) {
            tlIncident.setEndTime(DateUtil.now());
        }

        SysDictData sysDictData = new SysDictData();
        sysDictData.setDictType("incident_level");
        List<SysDictData>  list1 = sysDictDataService.selectDictDataList(sysDictData);
        Map<String,String> levelmap = new HashMap<>();
        for (SysDictData d:list1
        ) {
            levelmap.put(d.getDictValue(),d.getDictLabel());
        }

        sysDictData = new SysDictData();
        sysDictData.setDictType("tl_xun_area");
        list1 = sysDictDataService.selectDictDataList(sysDictData);
        Map<String,String> areamap = new HashMap<>();
        for (SysDictData d:list1
        ) {
            areamap.put(d.getDictValue(),d.getDictLabel());
        }
        sysDictData = new SysDictData();
        sysDictData.setDictType("incident_status");
        list1 = sysDictDataService.selectDictDataList(sysDictData);
        Map<String,String> statusmap = new HashMap<>();
        for (SysDictData d:list1
        ) {
            statusmap.put(d.getDictValue(),d.getDictLabel());
        }


        List<TlIncident> list = tlIncidentService.selectTlIncidentList(tlIncident);
        List<Map<String,Object>> datap = new ArrayList<>();
        for (TlIncident incident : list) {
            Map<String,Object> p = new HashMap<>();
            p.put("name",incident.getName());
            p.put("detail",incident.getDetail());
            p.put("incLevel",levelmap.get(incident.getIncLevel()));
            p.put("happenTime",DateUtil.formatDate(incident.getHappenTime()) );
            p.put("area",areamap.get(incident.getArea()));
            p.put("address",incident.getAddress());
            p.put("acceptUser",incident.getAcceptUserName());
            p.put("createByName",incident.getCreateByName());
            p.put("status",statusmap.get(incident.getStatus()));
            p.put("dealUser",incident.getDealUserName());
            p.put("dealTime",DateUtil.formatDate(incident.getDealTime()));


            if (StrUtil.isNotBlank(incident.getPics())) {
//                incident.setPics(StrUtil.subBefore(incident.getPics(), ",", false));
                List<String> pics = StrUtil.split(incident.getPics(), ",");
                List<String> pics1 = StrUtil.split(incident.getDealPics(), ",");
                pics.addAll(pics1);

                List<byte[]> picbytes = pics.stream().map(s -> {
                    byte[] a= {};
                    try {
                        a = ImageUtils.getImage(RuoYiConfig.getDownloadPath() + s);
                    }catch (Exception e){

                    }
                    return a;
                }).collect(Collectors.toList());
                p.put("images",picbytes);
            }
            datap.add(p);
        }
        Map<String,Object> data = new HashMap<>();
        data.put("list",datap);

        exportExcel("巡检事件_"+DateUtil.current(),"location",data,response);
//        exportExcel()
//        ExcelUtil<TlIncident> util = new ExcelUtil<TlIncident>(TlIncident.class);
//        util.exportExcel(response, list, "巡检事件管理数据");
    }

    /**
     * 获取巡检事件管理详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(tlIncidentService.selectTlIncidentById(id));
    }

    /**
     * 新增巡检事件管理
     */
    @Log(title = "巡检事件管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TlIncident tlIncident) {
        tlIncident.setCreateBy(getUsername());
        tlIncident.setCreateByName(getLoginUser().getUser().getNickName());
        tlIncident.setStatus("1");
        return toAjax(tlIncidentService.insertTlIncident(tlIncident));
    }

    /**
     * 修改巡检事件管理
     */
    @Log(title = "巡检事件管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TlIncident tlIncident) {
        return toAjax(tlIncidentService.updateTlIncident(tlIncident));
    }

    @Log(title = "巡检事件管理-处理", businessType = BusinessType.UPDATE)
    @PutMapping("/deal")
    public AjaxResult deal(@RequestBody TlIncident tlIncident) {
        if (ObjectUtil.isNull(tlIncident.getId()) || StrUtil.isBlank(tlIncident.getStatus())) {
            return AjaxResult.error("缺少必要参数！");
        }
        TlIncident save = new TlIncident();
        save.setId(tlIncident.getId());
        save.setStatus(tlIncident.getStatus());
        save.setDealUser(getUsername());
        save.setDealUserName(getLoginUser().getUser().getNickName());
        save.setDealTime(new Date());
        return toAjax(tlIncidentService.updateTlIncident(save));
    }

    @Log(title = "巡检事件管理-处理备注和图片", businessType = BusinessType.UPDATE)
    @PutMapping("/dealRemark")
    public AjaxResult dealRemark(@RequestBody TlIncident tlIncident) {
        if (ObjectUtil.isNull(tlIncident.getId())) {
            return AjaxResult.error("缺少必要参数！");
        }
        TlIncident save = new TlIncident();
        save.setId(tlIncident.getId());
        save.setDealRemark(tlIncident.getDealRemark());
        save.setDealPics(tlIncident.getDealPics());
        save.setDealTime(new Date());
        return toAjax(tlIncidentService.updateTlIncident(save));
    }

    /**
     * 删除巡检事件管理
     */
    @Log(title = "巡检事件管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(tlIncidentService.deleteTlIncidentByIds(ids));
    }


}
