package com.ruoyi.web.controller.qdtl;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.qdtl.domain.TlIncidentFault;
import com.ruoyi.qdtl.service.ITlIncidentFaultService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 应急故障事件管理Controller
 *
 * @author ruoyi
 * @date 2022-08-09
 */
@RestController
@RequestMapping("/qdtl/fault")
public class TlIncidentFaultController extends BaseController {
    @Autowired
    private ITlIncidentFaultService tlIncidentFaultService;

    /**
     * 查询应急故障事件管理列表
     */
    @GetMapping("/list")
    public TableDataInfo list(TlIncidentFault tlIncidentFault) {
        startPage();
        List<TlIncidentFault> list = tlIncidentFaultService.selectTlIncidentFaultList(tlIncidentFault);
        return getDataTable(list);
    }

    /**
     * 导出应急故障事件管理列表
     */
    @Log(title = "应急故障事件管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TlIncidentFault tlIncidentFault) {
        List<TlIncidentFault> list = tlIncidentFaultService.selectTlIncidentFaultList(tlIncidentFault);
        ExcelUtil<TlIncidentFault> util = new ExcelUtil<TlIncidentFault>(TlIncidentFault.class);
        util.exportExcel(response, list, "应急故障事件管理数据");
    }

    /**
     * 获取应急故障事件管理详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(tlIncidentFaultService.selectTlIncidentFaultById(id));
    }

    /**
     * 新增应急故障事件管理
     */
    @Log(title = "应急故障事件管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TlIncidentFault tlIncidentFault) {
        return toAjax(tlIncidentFaultService.insertTlIncidentFault(tlIncidentFault));
    }

    /**
     * 修改应急故障事件管理
     */
    @Log(title = "应急故障事件管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TlIncidentFault tlIncidentFault) {
        return toAjax(tlIncidentFaultService.updateTlIncidentFault(tlIncidentFault));
    }

    /**
     * 删除应急故障事件管理
     */
    @Log(title = "应急故障事件管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(tlIncidentFaultService.deleteTlIncidentFaultByIds(ids));
    }
}
