package com.ruoyi.web.controller.qdtl;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.qdtl.domain.TlInspectionDevice;
import com.ruoyi.qdtl.service.ITlInspectionDeviceService;
import com.ruoyi.system.service.ISysRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Set;

/**
 * 巡检设备管理Controller
 *
 * @author ruoyi
 * @date 2022-02-22
 */
@RestController
@RequestMapping("/qdtl/device")
public class TlInspectionDeviceController extends BaseController {
    @Autowired
    private ITlInspectionDeviceService tlInspectionDeviceService;
    @Autowired
    private ISysRoleService sysRoleService;

    /**
     * 查询巡检设备管理列表
     */
    @GetMapping("/list")
    public TableDataInfo list(TlInspectionDevice tlInspectionDevice) {
        Set<String> roleSet = sysRoleService.selectRolePermissionByUserId(getUserId());
        tlInspectionDevice.setAreaIds(roleSet);
        startPage();
        List<TlInspectionDevice> list = tlInspectionDeviceService.selectTlInspectionDeviceList(tlInspectionDevice);
        return getDataTable(list);
    }

    /**
     * 导出巡检设备管理列表
     */
    @Log(title = "巡检设备管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TlInspectionDevice tlInspectionDevice) {
        List<TlInspectionDevice> list = tlInspectionDeviceService.selectTlInspectionDeviceList(tlInspectionDevice);
        ExcelUtil<TlInspectionDevice> util = new ExcelUtil<TlInspectionDevice>(TlInspectionDevice.class);
        util.exportExcel(response, list, "巡检设备管理数据");
    }

    /**
     * 获取巡检设备管理详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(tlInspectionDeviceService.selectTlInspectionDeviceById(id));
    }

    /**
     * 新增巡检设备管理
     */
    @Log(title = "巡检设备管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TlInspectionDevice tlInspectionDevice) {
        return toAjax(tlInspectionDeviceService.insertTlInspectionDevice(tlInspectionDevice));
    }

    /**
     * 修改巡检设备管理
     */
    @Log(title = "巡检设备管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TlInspectionDevice tlInspectionDevice) {
        return toAjax(tlInspectionDeviceService.updateTlInspectionDevice(tlInspectionDevice));
    }

    /**
     * 删除巡检设备管理
     */
    @Log(title = "巡检设备管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{id}")
    public AjaxResult remove(@PathVariable Long id) {
        return toAjax(tlInspectionDeviceService.deleteTlInspectionDeviceById(id));
    }
}
