package com.ruoyi.web.controller.qdtl;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.qdtl.domain.TlInspectionLine;
import com.ruoyi.qdtl.domain.TlInspectionLocation;
import com.ruoyi.qdtl.service.ITlInspectionLineService;
import com.ruoyi.qdtl.service.ITlInspectionLocationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 巡检线路管理Controller
 *
 * @author ruoyi
 * @date 2022-03-03
 */
@RestController
@RequestMapping("/qdtl/line")
public class TlInspectionLineController extends BaseController {
    @Autowired
    private ITlInspectionLineService tlInspectionLineService;
    @Autowired
    private ITlInspectionLocationService tlInspectionLocationService;

    /**
     * 查询巡检线路管理列表
     */
    @GetMapping("/list")
    public TableDataInfo list(TlInspectionLine tlInspectionLine) {
        startPage();
        List<TlInspectionLine> list = tlInspectionLineService.selectTlInspectionLineList(tlInspectionLine);
        if (list.size() > 0) {
            List<Long> idCollect = list.stream().map(TlInspectionLine::getId).collect(Collectors.toList());
            // 查询所有的点位
            List<TlInspectionLocation> locations = tlInspectionLocationService.queryLocationByLineIds(idCollect);
            Map<Long, List<TlInspectionLocation>> collect = locations.stream().collect(Collectors.groupingBy(TlInspectionLocation::getLineId, Collectors.toList()));
            for (TlInspectionLine inspectionLine : list) {
                inspectionLine.setLocations(collect.get(inspectionLine.getId()));
                inspectionLine.setLineLocationNum(inspectionLine.getLocations() == null ? 0 : inspectionLine.getLocations().size());
            }
        }
        return getDataTable(list);
    }

    /**
     * 导出巡检线路管理列表
     */
    /*@PreAuthorize("@ss.hasPermi('qdtl:line:export')")
    @Log(title = "巡检线路管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TlInspectionLine tlInspectionLine)
    {
        List<TlInspectionLine> list = tlInspectionLineService.selectTlInspectionLineList(tlInspectionLine);
        ExcelUtil<TlInspectionLine> util = new ExcelUtil<TlInspectionLine>(TlInspectionLine.class);
        util.exportExcel(response, list, "巡检线路管理数据");
    }*/

    /**
     * 获取巡检线路管理详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        TlInspectionLine tlInspectionLine = tlInspectionLineService.selectTlInspectionLineById(id);
        // 查询线路的点位数据
        List<Long> ids = new ArrayList<>();
        ids.add(id);
        List<TlInspectionLocation> locations = tlInspectionLocationService.queryLocationByLineIds(ids);
        for (TlInspectionLocation location : locations) {
            // 查询点位的设备数据
        }
        return AjaxResult.success();
    }

    /**
     * 新增巡检线路管理
     */
    @Log(title = "巡检线路管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TlInspectionLine tlInspectionLine) {
        return toAjax(tlInspectionLineService.insertTlInspectionLine(tlInspectionLine));
    }

    /**
     * 修改巡检线路管理
     */
    @Log(title = "巡检线路管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TlInspectionLine tlInspectionLine) {
        return toAjax(tlInspectionLineService.updateTlInspectionLine(tlInspectionLine));
    }

    /**
     * 删除巡检线路管理
     */
    @Log(title = "巡检线路管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(tlInspectionLineService.deleteTlInspectionLineByIds(ids));
    }
}
