package com.ruoyi.web.controller.qdtl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.StrUtil;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.qdtl.domain.TlArea;
import com.ruoyi.qdtl.domain.TlInspectionDevice;
import com.ruoyi.qdtl.domain.TlInspectionLocation;
import com.ruoyi.qdtl.service.ITlAreaService;
import com.ruoyi.qdtl.service.ITlInspectionDeviceService;
import com.ruoyi.qdtl.service.ITlInspectionLocationService;
import com.ruoyi.system.service.ISysRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 巡检点管理Controller
 *
 * @author ruoyi
 * @date 2022-02-22
 */
@RestController
@RequestMapping("/qdtl/location")
public class TlInspectionLocationController extends BaseController {
    @Autowired
    private ITlInspectionLocationService tlInspectionLocationService;
    @Autowired
    private ITlInspectionDeviceService tlInspectionDeviceService;
    @Autowired
    private ITlAreaService tlAreaService;
    @Autowired
    private ISysRoleService sysRoleService;

    /**
     * 查询巡检点管理列表
     */
    @GetMapping("/list")
    public TableDataInfo list(TlInspectionLocation tlInspectionLocation) {
        Set<String> roleSet = sysRoleService.selectRolePermissionByUserId(getUserId());
        tlInspectionLocation.setAreaIds(roleSet);
        startPage();
        List<TlInspectionLocation> list = tlInspectionLocationService.selectTlInspectionLocationList(tlInspectionLocation);
        // 查询区域数据
        List<TlArea> areas = tlAreaService.selectTlAreaList(new TlArea());
        Map<Long, String> areaCollect = areas.stream().collect(Collectors.toMap(TlArea::getId, TlArea::getAreaName));

        for (TlInspectionLocation inspectionLocation : list) {
            List<TlInspectionDevice> devices = new ArrayList<>();
            if (StrUtil.isNotBlank(inspectionLocation.getDeviceIds())) {
                for (String s : inspectionLocation.getDeviceIds().split(",")) {
                    // 查询设备id所对应的名称
                    TlInspectionDevice tlInspectionDevice = tlInspectionDeviceService.selectTlInspectionDeviceById(Convert.toLong(s));
                    if (tlInspectionDevice != null) {
                        devices.add(tlInspectionDevice);
                    }
                }
            }
            inspectionLocation.setDeviceNames(devices);
            inspectionLocation.setDeviceNamesText(devices.stream().map(TlInspectionDevice::getDeviceName).collect(Collectors.joining(",")));
            inspectionLocation.setAreaName(areaCollect.getOrDefault(inspectionLocation.getAreaId(), ""));
        }
        return getDataTable(list);
    }

    /**
     * 导出巡检点管理列表
     */
    @Log(title = "巡检点管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TlInspectionLocation tlInspectionLocation) {
        List<TlInspectionLocation> list = tlInspectionLocationService.selectTlInspectionLocationList(tlInspectionLocation);
        ExcelUtil<TlInspectionLocation> util = new ExcelUtil<TlInspectionLocation>(TlInspectionLocation.class);
        util.exportExcel(response, list, "巡检点管理数据");
    }

    /**
     * 获取巡检点管理详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(tlInspectionLocationService.selectTlInspectionLocationById(id));
    }

    /**
     * 新增巡检点管理
     */
    @Log(title = "巡检点管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TlInspectionLocation tlInspectionLocation) {
        return toAjax(tlInspectionLocationService.insertTlInspectionLocation(tlInspectionLocation));
    }

    /**
     * 修改巡检点管理
     */
    @Log(title = "巡检点管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TlInspectionLocation tlInspectionLocation) {
        return toAjax(tlInspectionLocationService.updateTlInspectionLocation(tlInspectionLocation));
    }

    /**
     * 删除巡检点管理
     */
    @Log(title = "巡检点管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(tlInspectionLocationService.deleteTlInspectionLocationByIds(ids));
    }
}
