package com.ruoyi.web.controller.qdtl;

import cn.hutool.core.collection.CollUtil;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.qdtl.domain.TlArea;
import com.ruoyi.qdtl.domain.TlAreaMonitor;
import com.ruoyi.qdtl.domain.TlMonitor;
import com.ruoyi.qdtl.service.ITlAreaService;
import com.ruoyi.qdtl.service.ITlMonitorService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 监控设备管理Controller
 *
 * @author ruoyi
 * @date 2022-03-02
 */
@RestController
@RequestMapping("/qdtl/monitor")
public class TlMonitorController extends BaseController {
    @Autowired
    private ITlMonitorService tlMonitorService;
    @Autowired
    private ITlAreaService tlAreaService;

    /**
     * 查询监控设备管理列表
     */
    @GetMapping("/list")
    public AjaxResult list(TlMonitor tlMonitor) {
        List<TlMonitor> list = tlMonitorService.selectTlMonitorList(tlMonitor);
        Map<Long, List<TlMonitor>> map = list.stream().collect(Collectors.groupingBy(TlMonitor::getAreaId, Collectors.toList()));
        TlArea tlArea = new TlArea();
        tlArea.setAreaType("1");
        List<TlArea> areas = tlAreaService.selectTlAreaList(tlArea);
        List<TlAreaMonitor> areaMonitors = new ArrayList<>();
        for (TlArea area : areas) {
            TlAreaMonitor tlAreaMonitor = new TlAreaMonitor();
            tlAreaMonitor.setId(area.getId());
            tlAreaMonitor.setAreaName(area.getAreaName());
            tlAreaMonitor.setMonitors(CollUtil.defaultIfEmpty(map.get(area.getId()), new ArrayList<>()));
            areaMonitors.add(tlAreaMonitor);
        }
        return AjaxResult.success(areaMonitors);
    }

    /**
     * 导出监控设备管理列表
     */
    @Log(title = "监控设备管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TlMonitor tlMonitor) {
        List<TlMonitor> list = tlMonitorService.selectTlMonitorList(tlMonitor);
        ExcelUtil<TlMonitor> util = new ExcelUtil<TlMonitor>(TlMonitor.class);
        util.exportExcel(response, list, "监控设备管理数据");
    }

    /**
     * 获取监控设备管理详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(tlMonitorService.selectTlMonitorById(id));
    }

    /**
     * 新增监控设备管理
     */
    @Log(title = "监控设备管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TlMonitor tlMonitor) {
        return toAjax(tlMonitorService.insertTlMonitor(tlMonitor));
    }

    /**
     * 修改监控设备管理
     */
    @Log(title = "监控设备管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TlMonitor tlMonitor) {
        return toAjax(tlMonitorService.updateTlMonitor(tlMonitor));
    }

    /**
     * 删除监控设备管理
     */
    @Log(title = "监控设备管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(tlMonitorService.deleteTlMonitorByIds(ids));
    }

    @Log(title = "监控设备修改名称", businessType = BusinessType.UPDATE)
    @PutMapping("/editName")
    public AjaxResult editName(@RequestBody TlMonitor tlMonitor) {
        return toAjax(tlMonitorService.updateTlMonitorName(tlMonitor));
    }

    @GetMapping("/queryCamera")
    public AjaxResult queryCamera() {
        return AjaxResult.success(tlMonitorService.queryCamera());
    }
}
