package com.ruoyi.web.controller.qdtl;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.qdtl.domain.TlPatrolman;
import com.ruoyi.qdtl.service.ITlPatrolmanService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 巡检员Controller
 *
 * @author ruoyi
 * @date 2022-03-25
 */
@RestController
@RequestMapping("/qdtl/patrolman")
public class TlPatrolmanController extends BaseController {
    @Autowired
    private ITlPatrolmanService tlPatrolmanService;

    /**
     * 查询巡检员列表
     */
    @GetMapping("/list")
    public TableDataInfo list(TlPatrolman tlPatrolman) {
        startPage();
        List<TlPatrolman> list = tlPatrolmanService.selectTlPatrolmanList(tlPatrolman);
        return getDataTable(list);
    }

    /**
     * 获取所有的巡检人员
     *
     * @return
     */
    @GetMapping("/xunlist")
    public AjaxResult getXunList(TlPatrolman tlPatrolman) {
        return AjaxResult.success(tlPatrolmanService.selectTlPatrolmanList(tlPatrolman));
    }

    /**
     * 导出巡检员列表
     */
    @Log(title = "巡检员", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TlPatrolman tlPatrolman) {
        List<TlPatrolman> list = tlPatrolmanService.selectTlPatrolmanList(tlPatrolman);
        ExcelUtil<TlPatrolman> util = new ExcelUtil<TlPatrolman>(TlPatrolman.class);
        util.exportExcel(response, list, "巡检员数据");
    }

    /**
     * 获取巡检员详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(tlPatrolmanService.selectTlPatrolmanById(id));
    }

    /**
     * 新增巡检员
     */
    @Log(title = "巡检员", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TlPatrolman tlPatrolman) {
        return toAjax(tlPatrolmanService.insertTlPatrolman(tlPatrolman));
    }

    /**
     * 修改巡检员
     */
    @Log(title = "巡检员", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TlPatrolman tlPatrolman) {
        return toAjax(tlPatrolmanService.updateTlPatrolman(tlPatrolman));
    }

    /**
     * 人员报表
     *
     * @param cards
     * @return
     */
    @GetMapping(value = "/getPatrolmanStatic")
    public AjaxResult getPatrolmanStatic(@RequestParam String cards) {
        return AjaxResult.success(tlPatrolmanService.getPatrolmanStatic(cards));
    }
}
