package com.ruoyi.web.controller.qdtl;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.qdtl.domain.TlPlan;
import com.ruoyi.qdtl.service.ITlPlanService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 应急预案管理Controller
 *
 * @author ruoyi
 * @date 2022-08-09
 */
@RestController
@RequestMapping("/qdtl/yjPlan")
public class TlPlanController extends BaseController {
    @Autowired
    private ITlPlanService tlPlanService;

    /**
     * 查询应急预案管理列表
     */
    @GetMapping("/list")
    public TableDataInfo list(TlPlan tlPlan) {
        startPage();
        List<TlPlan> list = tlPlanService.selectTlPlanList(tlPlan);
        return getDataTable(list);
    }

    /**
     * 导出应急预案管理列表
     */
    @Log(title = "应急预案管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TlPlan tlPlan) {
        List<TlPlan> list = tlPlanService.selectTlPlanList(tlPlan);
        ExcelUtil<TlPlan> util = new ExcelUtil<TlPlan>(TlPlan.class);
        util.exportExcel(response, list, "应急预案管理数据");
    }

    /**
     * 获取应急预案管理详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(tlPlanService.selectTlPlanById(id));
    }

    /**
     * 新增应急预案管理
     */
    @Log(title = "应急预案管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TlPlan tlPlan) {
        return toAjax(tlPlanService.insertTlPlan(tlPlan));
    }

    /**
     * 修改应急预案管理
     */
    @Log(title = "应急预案管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TlPlan tlPlan) {
        return toAjax(tlPlanService.updateTlPlan(tlPlan));
    }

    /**
     * 删除应急预案管理
     */
    @Log(title = "应急预案管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(tlPlanService.deleteTlPlanByIds(ids));
    }
}
