package com.ruoyi.web.controller.qdtl;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.qdtl.domain.TlPlanRecord;
import com.ruoyi.qdtl.service.ITlPlanRecordService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 巡检计划记录Controller
 *
 * @author ruoyi
 * @date 2022-03-30
 */
@RestController
@RequestMapping("/qdtl/record")
public class TlPlanRecordController extends BaseController {
    @Autowired
    private ITlPlanRecordService tlPlanRecordService;

    /**
     * 查询巡检计划记录列表
     */
    @GetMapping("/list")
    public TableDataInfo list(TlPlanRecord tlPlanRecord) {
        startPage();
        List<TlPlanRecord> list = tlPlanRecordService.selectTlPlanRecordList(tlPlanRecord);
        return getDataTable(list);
    }

    /**
     * 导出巡检计划记录列表
     */
    @Log(title = "巡检计划记录", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TlPlanRecord tlPlanRecord) {
        List<TlPlanRecord> list = tlPlanRecordService.selectTlPlanRecordList(tlPlanRecord);
        ExcelUtil<TlPlanRecord> util = new ExcelUtil<TlPlanRecord>(TlPlanRecord.class);
        util.exportExcel(response, list, "巡检计划记录数据");
    }

    /**
     * 获取巡检计划记录详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(tlPlanRecordService.selectTlPlanRecordById(id));
    }

    /**
     * 新增巡检计划记录
     */
    @Log(title = "巡检计划记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TlPlanRecord tlPlanRecord) {
        return toAjax(tlPlanRecordService.insertTlPlanRecord(tlPlanRecord));
    }

    /**
     * 修改巡检计划记录
     */
    @Log(title = "巡检计划记录", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TlPlanRecord tlPlanRecord) {
        return toAjax(tlPlanRecordService.updateTlPlanRecord(tlPlanRecord));
    }

    /**
     * 删除巡检计划记录
     */
    @Log(title = "巡检计划记录", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(tlPlanRecordService.deleteTlPlanRecordByIds(ids));
    }
}
