package com.ruoyi.web.controller.qdtl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONObject;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.qdtl.domain.TlWorkPic;
import com.ruoyi.qdtl.service.ITlWorkPicService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 工作面貌图片管理Controller
 *
 * @author ruoyi
 * @date 2022-08-09
 */
@RestController
@RequestMapping("/qdtl/pic")
public class TlWorkPicController extends BaseController {
    @Autowired
    private ITlWorkPicService tlWorkPicService;

    /**
     * 查询工作面貌图片管理列表
     */
    @GetMapping("/list")
    public TableDataInfo list(TlWorkPic tlWorkPic) {
        startPage();
        List<TlWorkPic> list = tlWorkPicService.selectTlWorkPicList(tlWorkPic);
        return getDataTable(list);
    }

    /**
     * 导出工作面貌图片管理列表
     */
    @Log(title = "工作面貌图片管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TlWorkPic tlWorkPic) {
        List<TlWorkPic> list = tlWorkPicService.selectTlWorkPicList(tlWorkPic);
        ExcelUtil<TlWorkPic> util = new ExcelUtil<TlWorkPic>(TlWorkPic.class);
        util.exportExcel(response, list, "工作面貌图片管理数据");
    }

    /**
     * 获取工作面貌图片管理详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(tlWorkPicService.selectTlWorkPicById(id));
    }

    /**
     * 新增工作面貌图片管理
     */
    @Log(title = "工作面貌图片管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TlWorkPic tlWorkPic) {
        return toAjax(tlWorkPicService.insertTlWorkPic(tlWorkPic));
    }

    @Log(title = "工作面貌图片顺序调整", businessType = BusinessType.UPDATE)
    @PostMapping("/changeSortNo")
    public AjaxResult changeSortNo(@RequestBody JSONObject jsonObject) {
        Long idOne = jsonObject.getLong("idOne");
        Long idTwo = jsonObject.getLong("idTwo");
        boolean b = ObjectUtil.hasEmpty(idOne, idTwo);
        if (b) {
            return AjaxResult.error("调整顺序的2条数据不能为空");
        }
        return toAjax(tlWorkPicService.changeSortNo(idOne, idTwo));
    }

    /**
     * 修改工作面貌图片管理
     */
    @Log(title = "工作面貌图片管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TlWorkPic tlWorkPic) {
        return toAjax(tlWorkPicService.updateTlWorkPic(tlWorkPic));
    }

    /**
     * 删除工作面貌图片管理
     */
    @Log(title = "工作面貌图片管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(tlWorkPicService.deleteTlWorkPicByIds(ids));
    }
}
