package com.ruoyi.web.controller.task;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.NumberUtil;
import com.ruoyi.qdtl.domain.TlElectricCarMileage;
import com.ruoyi.qdtl.domain.TlElectricDeviceLocation;
import com.ruoyi.qdtl.service.ITlElectricCarMileageService;
import com.ruoyi.qdtl.service.ITlElectricDeviceLocationService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: 电瓶车定位相关任务
 * @Author: huangcheng
 * @Date: 2022/6/23
 * @Version V1.0
 */
@Component("electricCarTask")
@Slf4j
public class ElectricCarTask {

    @Autowired
    private ITlElectricDeviceLocationService electricDeviceLocationService;
    @Autowired
    private ITlElectricCarMileageService electricCarMileageService;

    /**
     * 同步电动车位置信息
     */
    public void syncLocation() {
        electricDeviceLocationService.syncDeviceLocation();
    }

    /**
     * 统计每辆车每天的里程
     */
    public void staticCarDayMileage() {
        // 统计当天的里程数据。根据时间排序，计算第一条和最后一条的数据差
        String today = DateUtil.today();
        // 根据日期查询记录
        List<TlElectricDeviceLocation> list = electricDeviceLocationService.queryListByDate(today);
        // 根据设备分组
        Map<String, List<TlElectricDeviceLocation>> collect = list.stream().collect(Collectors.groupingBy(TlElectricDeviceLocation::getImei));
        for (Map.Entry<String, List<TlElectricDeviceLocation>> stringListEntry : collect.entrySet()) {
            String imei = stringListEntry.getKey();
            List<TlElectricDeviceLocation> value = stringListEntry.getValue();
            // 计算中间的里程
            BigDecimal mileage;
            if (value != null && value.size() > 0) {
                mileage = NumberUtil.sub(NumberUtil.round(value.get(value.size() - 1).getMileage(), 1), NumberUtil.round(value.get(0).getMileage(), 1));
                // 先根据imei和日期查询
                TlElectricCarMileage electricCarMileage = new TlElectricCarMileage();
                electricCarMileage.setImei(imei);
                electricCarMileage.setDays(today);
                List<TlElectricCarMileage> tlElectricCarMileages = electricCarMileageService.selectTlElectricCarMileageList(electricCarMileage);
                electricCarMileage.setDeviceName(value.get(0).getDeviceName());
                electricCarMileage.setMileage(Convert.toStr(mileage));
                if (tlElectricCarMileages == null || tlElectricCarMileages.size() == 0) {
                    // 新增
                    electricCarMileageService.insertTlElectricCarMileage(electricCarMileage);
                } else {
                    // 更新
                    electricCarMileage.setId(tlElectricCarMileages.get(0).getId());
                    electricCarMileageService.updateTlElectricCarMileage(electricCarMileage);
                }
            }
        }
    }

}
