package com.ruoyi.web.controller.task;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.ruoyi.qdtl.domain.TlInspectionDevice;
import com.ruoyi.qdtl.domain.TlInspectionLocation;
import com.ruoyi.qdtl.domain.TlInspectionLocationLog;
import com.ruoyi.qdtl.domain.TlPatrolman;
import com.ruoyi.qdtl.domain.TlPlanRecord;
import com.ruoyi.qdtl.service.IQdCommonService;
import com.ruoyi.qdtl.service.ITlInspectionDeviceService;
import com.ruoyi.qdtl.service.ITlInspectionLocationLogService;
import com.ruoyi.qdtl.service.ITlInspectionLocationService;
import com.ruoyi.qdtl.service.ITlPatrolmanService;
import com.ruoyi.qdtl.service.ITlPlanRecordService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;

/**
 * @Description: TODO
 * @Author: huangcheng
 * @Date: 2022/3/7
 * @Version V1.0
 */
@Component("xunTask")
@Slf4j
public class XunTask {

    @Autowired
    private IQdCommonService qdCommonService;
    @Autowired
    private ITlInspectionDeviceService tlInspectionDeviceService;
    @Autowired
    private ITlInspectionLocationService tlInspectionLocationService;
    @Autowired
    private ITlInspectionLocationLogService tlInspectionLocationLogService;
    @Autowired
    private ITlPatrolmanService tlPatrolmanService;
    @Autowired
    private ITlPlanRecordService planRecordService;

    /**
     * 同步设备列表
     */
    public void syncDevice() {
        JSONArray deviceList = qdCommonService.getDevice();
        if (deviceList != null && deviceList.size() > 0) {
            // 获取当前需要更新的版本号
            int version = qdCommonService.getVersion("tl:version:devicce");
            for (Object o : deviceList) {
                JSONObject jsonObject = JSONUtil.parseObj(o);
                String thirdId = jsonObject.getStr("id");
                String name = jsonObject.getStr("name");
                String code = jsonObject.getStr("code");
                Long areaId = jsonObject.getLong("areaId");
                // 根据thirdId查询是否存在
                TlInspectionDevice tlInspectionDevice = tlInspectionDeviceService.queryByThirdId(thirdId);
                if (tlInspectionDevice != null) {
                    tlInspectionDevice.setDeviceCode(code);
                    tlInspectionDevice.setDeviceName(name);
                    tlInspectionDevice.setAreaId(areaId);
                    tlInspectionDevice.setVersion(version);
                    tlInspectionDeviceService.updateTlInspectionDevice(tlInspectionDevice);
                } else {
                    tlInspectionDevice = new TlInspectionDevice();
                    tlInspectionDevice.setDeviceCode(code);
                    tlInspectionDevice.setDeviceName(name);
                    tlInspectionDevice.setAreaId(areaId);
                    tlInspectionDevice.setVersion(version);
                    tlInspectionDevice.setThirdId(thirdId);
                    tlInspectionDeviceService.insertTlInspectionDevice(tlInspectionDevice);
                }
            }
            // 删除非此版本号的数据
            tlInspectionDeviceService.deleteLocationByVersion(version);
        }
    }

    /**
     * 同步巡检点
     */
    public void syncCheckpoint() {
        JSONArray checkpoints = qdCommonService.getCheckpoint();
        if (checkpoints != null && checkpoints.size() > 0) {
            // 获取当前需要更新的版本号
            int version = qdCommonService.getVersion("tl:version:checkpoint");
            for (Object checkpoint : checkpoints) {
                JSONObject jsonObject = JSONUtil.parseObj(checkpoint);
                String thirdId = jsonObject.getStr("id");
                String name = jsonObject.getStr("name");
                String code = jsonObject.getStr("card");
                // 根据thirdId查询是否存在
                TlInspectionLocation tlInspectionLocation = tlInspectionLocationService.queryByThirdId(thirdId);
                if (tlInspectionLocation != null) {
                    tlInspectionLocation.setLocationCode(code);
                    tlInspectionLocation.setLocationName(name);
                    tlInspectionLocation.setVersion(version);
                    tlInspectionLocationService.updateTlInspectionLocation(tlInspectionLocation);
                } else {
                    tlInspectionLocation = new TlInspectionLocation();
                    tlInspectionLocation.setLocationCode(code);
                    tlInspectionLocation.setLocationName(name);
                    tlInspectionLocation.setVersion(version);
                    tlInspectionLocation.setThirdId(thirdId);
                    tlInspectionLocationService.insertTlInspectionLocation(tlInspectionLocation);
                }
            }
            // 删除非此版本号的数据
            tlInspectionLocationService.deleteLocationByVersion(version);
        }
    }

    /**
     * 同步巡检记录
     */
    public void syncCheckpointLog() {
        // 获取目前数据库中最后的记录事件
        Date maxDate = tlInspectionLocationLogService.queryMaxDate();
        long startTime;
        long endTime = DateUtil.current();
        if (maxDate == null) {
            startTime = DateUtil.offsetDay(new Date(), -20).getTime();
        } else {
            // 判断最后的日期和当前是否大于1个月
            long between = DateUtil.between(maxDate, new Date(), DateUnit.DAY);
            if (between >= 30) {
                startTime = DateUtil.offsetDay(new Date(), -29).getTime();
            } else {
                startTime = DateUtil.offsetSecond(maxDate, 1).getTime();
            }
        }
        JSONArray checkpointLogs = qdCommonService.getCheckpointLog(startTime, endTime);
        if (checkpointLogs != null && checkpointLogs.size() > 0) {
            for (Object checkpointLog : checkpointLogs) {
                JSONObject jsonObject = JSONUtil.parseObj(checkpointLog);
                TlInspectionLocationLog convert = Convert.convert(TlInspectionLocationLog.class, checkpointLog);
                convert.setCreateTime(DateUtil.date(jsonObject.getLong("createTime")));
                // 查询该点位的经纬度
                TlInspectionLocation tlInspectionLocation = tlInspectionLocationService.queryByThirdId(convert.getCheckpointId());
                if (tlInspectionLocation != null && StrUtil.isNotBlank(tlInspectionLocation.getLnglat())) {
                    convert.setFence(tlInspectionLocation.getLnglat());
                } else {
                    log.error("没有通过CheckpointId找到相应的巡检点信息{}", convert.getCheckpointId());
                }
                tlInspectionLocationLogService.insertTlInspectionLocationLog(convert);

                // 更新到计划中
                if (StrUtil.isNotBlank(convert.getPatrolmanCard())) {
                    // 根据当前时间查询符合的计划数据
                    TlPlanRecord tlPlanRecord = new TlPlanRecord();
                    tlPlanRecord.setCard(convert.getPatrolmanCard());
                    tlPlanRecord.setCheckpointCard(convert.getCheckpointCard());
                    tlPlanRecord.setDeviceCode(convert.getDeviceCode());
                    tlPlanRecord.setFence(tlInspectionLocation.getLnglat());
                    tlPlanRecord.setAreaId(tlInspectionLocation.getAreaId());
                    tlPlanRecord.setCreateTime(convert.getCreateTime());
                    planRecordService.updateRecordByDateAndCard(tlPlanRecord);
                }
            }
        }
    }

    /**
     * 同步巡检轨迹
     */
    public void syncTrail() {
        // 暂无
    }

    /**
     * 同步巡检员
     */
    public void syncPatrolman() {
        JSONArray patrolmans = qdCommonService.getPatrolman();
        if (patrolmans != null && patrolmans.size() > 0) {
            // 获取当前需要更新的版本号
            int version = qdCommonService.getVersion("tl:version:patrolman");
            for (Object patrolman : patrolmans) {
                JSONObject jsonObject = JSONUtil.parseObj(patrolman);
                String thirdId = jsonObject.getStr("id");
                String name = jsonObject.getStr("name");
                String card = jsonObject.getStr("card");
                String areaId = jsonObject.getStr("areaId");
                String areaName = jsonObject.getStr("areaName");
                // 根据thirdId查询是否存在
                TlPatrolman tlPatrolman = tlPatrolmanService.queryByThirdId(thirdId);
                if (tlPatrolman != null) {
                    tlPatrolman.setName(name);
                    tlPatrolman.setCard(card);
                    tlPatrolman.setAreaId(areaId);
                    tlPatrolman.setAreaName(areaName);
                    tlPatrolman.setVersion(version);
                    tlPatrolmanService.updateTlPatrolman(tlPatrolman);
                } else {
                    tlPatrolman = new TlPatrolman();
                    tlPatrolman.setName(name);
                    tlPatrolman.setCard(card);
                    tlPatrolman.setAreaId(areaId);
                    tlPatrolman.setAreaName(areaName);
                    tlPatrolman.setVersion(version);
                    tlPatrolman.setThirdId(thirdId);
                    tlPatrolmanService.insertTlPatrolman(tlPatrolman);
                }
            }
            // 删除非此版本号的数据
            tlPatrolmanService.deletePatrolmanByVersion(version);
        }
    }

}
