package com.ruoyi.qdtl.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.qdtl.domain.*;
import com.ruoyi.qdtl.mapper.TlInspectionLocationLogMapper;
import com.ruoyi.qdtl.mapper.TlInspectionPlanMapper;
import com.ruoyi.qdtl.mapper.TlPlanRecordMapper;
import com.ruoyi.qdtl.service.IQdCommonService;
import com.ruoyi.qdtl.service.ITlAreaService;
import com.ruoyi.qdtl.service.ITlInspectionDeviceService;
import com.ruoyi.qdtl.service.ITlInspectionLineLocationService;
import com.ruoyi.qdtl.service.ITlInspectionLineService;
import com.ruoyi.qdtl.service.ITlInspectionLocationService;
import com.ruoyi.qdtl.service.ITlInspectionPlanService;
import com.ruoyi.qdtl.service.ITlPatrolmanService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @Description: TODO
 * @Author: huangcheng
 * @Date: 2022/3/7
 * @Version V1.0
 */
@Service
@Slf4j
public class QdCommonServiceImpl implements IQdCommonService {

    @Value("${third.xuntoken}")
    private String xuntokenUrl;
    @Value("${third.xundevice}")
    private String xundeviceUrl;
    @Value("${third.xuncheckpointLog}")
    private String xuncheckpointLogUrl;
    @Value("${third.xuncheckpoint}")
    private String xuncheckpointUrl;
    @Value("${third.xunpatrolman}")
    private String xunpatrolmanUrl;
    @Value("${third.xuntrail}")
    private String xuntrailUrl;
    @Value("${third.electricCarsOpenApi}")
    private String electricCarsOpenApiUrl;

    @Autowired
    private RedisCache redisCache;
    @Autowired
    private ITlInspectionLocationService tlInspectionLocationService;
    @Autowired
    private ITlInspectionDeviceService tlInspectionDeviceService;
    @Autowired
    private ITlPatrolmanService tlPatrolmanService;
    @Autowired
    private ITlInspectionPlanService inspectionPlanService;
    @Autowired
    private ITlInspectionLineLocationService inspectionLineLocationService;
    @Autowired
    private ITlInspectionLineService inspectionLineService;
    @Autowired
    private TlInspectionPlanMapper tlInspectionPlanMapper;
    @Autowired
    private TlInspectionLocationLogMapper tlInspectionLocationLogMapper;
    @Autowired
    private TlPlanRecordMapper planRecordMapper;
    @Autowired
    private ITlAreaService areaService;

    /**
     * 获取巡检接口的token
     *
     * @return
     */
    @Override
    public String getXunToken() {
        String token = redisCache.getCacheObject("xun:token");
        if (StrUtil.isNotBlank(token)) {
            return token;
        }
        Map<String, Object> params = new HashMap<>();
        params.put("username", "15190837633");
        params.put("password", "e10adc3949ba59abbe56e057f20f883e");
        String post = HttpUtil.post(xuntokenUrl, params);
        JSONObject postJson = JSONUtil.parseObj(post);
        if (200 == postJson.getInt("code")) {
            token = postJson.getStr("data");
            // 存入缓存。暂定为1小时过期
            redisCache.setCacheObject("xun:token", token, 1, TimeUnit.HOURS);
        } else {
            log.error("获取xun接口的token失败:{}", postJson);
            throw new ServiceException("获取xun接口的token失败");
        }
        return token;
    }

    /**
     * 设备列表
     *
     * @return
     */
    @Override
    public JSONArray getDevice() {
        String get = HttpRequest.get(xundeviceUrl).header("Authorization", this.getXunToken()).execute().body();
        JSONObject getJson = JSONUtil.parseObj(get);
        if (200 == getJson.getInt("code")) {
            JSONArray data = getJson.getJSONArray("data");
            return data;
        }
        log.error("调用我爱巡接口报错：{}", getJson);
        return new JSONArray();
    }

    /**
     * 查询巡检记录
     *
     * @param startTime 时间戳（秒）
     * @param endTime
     * @return
     */
    @Override
    @SneakyThrows
    public JSONArray getCheckpointLog(long startTime, long endTime) {
        if (ObjectUtil.isNull(endTime)) {
            endTime = DateUtil.current();
        }
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("startTime", startTime);
        paramMap.put("endTime", endTime);
        paramMap.put("pageSize", 5000);
        log.info("查询巡检记录--->请求参数：{}", paramMap);
        String get = HttpRequest.get(xuncheckpointLogUrl).form(paramMap).header("Authorization", this.getXunToken()).execute().body();
        JSONObject getJson = JSONUtil.parseObj(get);
        if (200 == getJson.getInt("code")) {
            JSONObject data = getJson.getJSONObject("data");
            if (data != null) {
                return data.getJSONArray("rows");
            }
        }
        log.error("调用我爱巡接口报错：{}", getJson);
        return null;
    }

    /**
     * 点位列表
     *
     * @return
     */
    @Override
    public JSONArray getCheckpoint() {
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("pageSize", 5000);
        String get = HttpRequest.get(xuncheckpointUrl).form(paramMap).header("Authorization", this.getXunToken()).execute().body();
        JSONObject getJson = JSONUtil.parseObj(get);
        if (200 == getJson.getInt("code")) {
            JSONObject data = getJson.getJSONObject("data");
            if (data != null) {
                return data.getJSONArray("rows");
            }
        }
        log.error("调用我爱巡接口报错：{}", getJson);
        return null;
    }

    @Override
    public JSONArray getPatrolman() {
        String get = HttpRequest.get(xunpatrolmanUrl).header("Authorization", this.getXunToken()).execute().body();
        JSONObject getJson = JSONUtil.parseObj(get);
        if (200 == getJson.getInt("code")) {
            return getJson.getJSONArray("data");
        }
        log.error("调用我爱巡接口报错：{}", getJson);
        return null;
    }

    /**
     * 查询设备运行轨迹
     *
     * @param startTime
     * @param endTime
     * @param deviceId
     * @return
     */
    @Override
    public JSONArray getTrail(long startTime, long endTime, String deviceId) {
        if (ObjectUtil.isNull(endTime)) {
            endTime = DateUtil.current();
        }
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("startTime", startTime);
        paramMap.put("endTime", endTime);
        paramMap.put("deviceId", deviceId);
        String get = HttpRequest.get(xuntrailUrl).form(paramMap).header("Authorization", this.getXunToken()).execute().body();
        return JSONUtil.parseArray(get);
    }

    @Override
    public int getVersion(String key) {
        boolean hasKey = redisCache.hasKey(key);
        if (!hasKey) {
            // 查询数据库中最大的
            int maxVsersion = 0;
            if (key.equals("tl:version:checkpoint")) {
                maxVsersion = tlInspectionLocationService.queryMaxVsersion();
            } else if (key.equals("tl:version:devicce")) {
                maxVsersion = tlInspectionDeviceService.queryMaxVsersion();
            } else if (key.equals("tl:version:patrolman")) {
                maxVsersion = tlPatrolmanService.queryMaxVsersion();
            }
            redisCache.setCacheObject(key, maxVsersion);
        }
        return Convert.toInt(redisCache.incr(key, 1));
    }

    /**
     * 获取电动车定位平台token
     *
     * @return
     */
    @Override
    public String getElectricCarsToken() {
        String token = redisCache.getCacheObject("electric:token");
        if (StrUtil.isBlank(token)) {
            Map<String, Object> paramMap = new HashMap<>();
            // 公共参数
            paramMap.put("app_key", "8FB345B8693CCD006879C3CE09549C82");
            paramMap.put("v", "0.9");
            paramMap.put("timestamp", DateUtil.now());
            paramMap.put("sign_method", "md5");
            paramMap.put("format", "json");
            paramMap.put("method", "jimi.oauth.token.get");
            // 私有参数
            paramMap.put("user_id", "启东市铁路巡逻电瓶车");
            paramMap.put("user_pwd_md5", "62ea9d1a6f6cfe766c69656c4869a819");
            paramMap.put("expires_in", "600");
            String post = HttpUtil.post(electricCarsOpenApiUrl, paramMap);
            JSONObject postJson = JSONUtil.parseObj(post);
            if (0 == postJson.getInt("code")) {
                token = postJson.getJSONObject("result").getStr("accessToken");
                // 存入缓存
                redisCache.setCacheObject("electric:token", token, 600, TimeUnit.SECONDS);
            } else {
                log.error("获取电动车定位平台接口的token失败:{}", postJson);
                throw new ServiceException("获取电动车定位平台接口的token失败");
            }
        }
        return token;
    }

    /**
     * 获取设备列表
     *
     * @return
     */
    @Override
    public List<TlElectricDevice> getElectricCarsDeviceList() {
        Map<String, Object> paramMap = new HashMap<>();
        // 公共参数
        paramMap.put("app_key", "8FB345B8693CCD006879C3CE09549C82");
        paramMap.put("v", "0.9");
        paramMap.put("timestamp", DateUtil.now());
        paramMap.put("sign_method", "md5");
        paramMap.put("format", "json");
        paramMap.put("method", "jimi.user.device.list");
        // 私有参数
        paramMap.put("target", "启东市铁路巡逻电瓶车");
        paramMap.put("access_token", this.getElectricCarsToken());
        String post = HttpUtil.post(electricCarsOpenApiUrl, paramMap);
        JSONObject postJson = JSONUtil.parseObj(post);
        if (0 == postJson.getInt("code")) {
            return JSONUtil.toList(postJson.getJSONArray("result"), TlElectricDevice.class);
        } else {
            log.error("获取电动车定位平台接口的设备列表失败:{}", postJson);
            throw new ServiceException("获取电动车定位平台接口的设备列表失败");
        }
    }

    /**
     * 获取设备定位最新数据
     *
     * @return
     */
    @Override
    public List<TlElectricDeviceLocation> getElectricCarsDeviceLocationList() {
        Map<String, Object> paramMap = new HashMap<>();
        // 公共参数
        paramMap.put("app_key", "8FB345B8693CCD006879C3CE09549C82");
        paramMap.put("v", "0.9");
        paramMap.put("timestamp", DateUtil.now());
        paramMap.put("sign_method", "md5");
        paramMap.put("format", "json");
        paramMap.put("method", "jimi.user.device.location.list");
        // 私有参数
        paramMap.put("target", "启东市铁路巡逻电瓶车");
        paramMap.put("access_token", this.getElectricCarsToken());
        String post = HttpUtil.post(electricCarsOpenApiUrl, paramMap);
        JSONObject postJson = JSONUtil.parseObj(post);
        if (0 == postJson.getInt("code")) {
            return JSONUtil.toList(postJson.getJSONArray("result"), TlElectricDeviceLocation.class);
        } else {
            log.error("获取电动车定位平台接口的设备最新位置失败:{}", postJson);
            throw new ServiceException("获取电动车定位平台接口的设备最新位置失败");
        }
    }

    @Override
    public Map<String, Object> monthStatic(Long areaId, String startDate, String endDate) {
        Map<String, Object> data = new HashMap<>();

        // 查询所有的线路点位
        List<TlInspectionLineLocation> lineLocations = inspectionLineLocationService.selectTlInspectionLineLocationList(new TlInspectionLineLocation());
        Map<Long, List<TlInspectionLineLocation>> lineLocationsCollect = lineLocations.stream().collect(Collectors.groupingBy(TlInspectionLineLocation::getLineId));
        // 查询线路数据
        TlInspectionLine line = new TlInspectionLine();
        line.setAreaId(areaId);
        List<TlInspectionLine> lines = inspectionLineService.selectTlInspectionLineList(line);
        for (TlInspectionLine tlInspectionLine : lines) {
            tlInspectionLine.setLineLocations(lineLocationsCollect.getOrDefault(tlInspectionLine.getId(), new ArrayList<>()));
        }
        Map<Long, Integer> LineLocationCount = lines.stream().collect(Collectors.toMap(TlInspectionLine::getId, a -> a.getLineLocations().size()));
        // 查询计划相关
        List<TlInspectionPlan> list = tlInspectionPlanMapper.queryByDatePeriod(startDate, endDate);
        List<DateTime> dateTimes = DateUtil.rangeToList(DateUtil.parseDate(startDate), DateUtil.parseDate(endDate), DateField.DAY_OF_YEAR);
        // 查询areaId对应我爱巡后台是哪个区域
        String areaName = "";
        if (ObjectUtil.isNotEmpty(areaId)) {
            TlArea tlArea = areaService.selectTlAreaById(areaId);
            if (tlArea != null) {
                areaName = tlArea.getAreaName();
            }
        }
        // 查询巡更记录表
        List<TlInspectionLocationLog> locationLogList = tlInspectionLocationLogMapper.queryDistanceByDateRange(areaName, startDate, endDate);
        Map<String, Long> locationLogCollectCount = locationLogList.stream().collect(Collectors.groupingBy(a -> DateUtil.formatDate(a.getCreateTime()), Collectors.counting()));
        // 查询计划中的数据
        List<TlPlanRecord> planRecords = planRecordMapper.selectTlPlanRecordListByDatePeriod(areaId, startDate, endDate);
        Map<String, Long> unPlanRecordsCollect = planRecords.stream().filter(a -> StrUtil.isBlank(a.getFence())).collect(Collectors.groupingBy(TlPlanRecord::getPlanDate, Collectors.counting()));
        Map<String, Long> planRecordsCollect = planRecords.stream().collect(Collectors.groupingBy(TlPlanRecord::getPlanDate, Collectors.counting()));

        List<XunStaticMonth> result = new ArrayList<>();
        for (DateTime existTime : dateTimes) {
            XunStaticMonth xunStaticMonth = new XunStaticMonth();
            // 计划巡更点总数
            int planLocationTotal = 0;
            for (TlInspectionPlan inspectionPlan : list) {
                // 每一个计划所有的日期
                String weeks = inspectionPlan.getWeeks();
                int week = DateUtil.dayOfWeek(existTime);
                if (CollUtil.contains(StrUtil.split(weeks, ","), Convert.toStr(week))) {
                    // 计划巡更点数
                    planLocationTotal += inspectionPlan.getXunNum() * LineLocationCount.getOrDefault(inspectionPlan.getLineId(), 0);
                }
            }
            xunStaticMonth.setStaticDate(DateUtil.formatDate(existTime));
            xunStaticMonth.setPlanLocationTotal(planLocationTotal);
            xunStaticMonth.setActualXunTotal(locationLogCollectCount.getOrDefault(DateUtil.formatDate(existTime), 0L));
            xunStaticMonth.setUnXunTotal(unPlanRecordsCollect.getOrDefault(DateUtil.formatDate(existTime), 0L));
            Long xun = planRecordsCollect.getOrDefault(DateUtil.formatDate(existTime), 0L);
            xunStaticMonth.setXunTotal(xun);
            xunStaticMonth.setUnXunPercent(xun == 0L ? new BigDecimal(0) : NumberUtil.mul(NumberUtil.div(xunStaticMonth.getUnXunTotal(), xun, 4), 100));
            result.add(xunStaticMonth);
        }
        data.put("xunList", result);
        // 本月计划巡更点总数
        Integer planXunLocationTotal = result.stream().mapToInt(XunStaticMonth::getPlanLocationTotal).sum();
        data.put("planXunLocationTotal", planXunLocationTotal);
        // 本月漏巡更点总数
        int unXunTotal = result.stream().mapToInt(a -> Convert.toInt(a.getUnXunTotal())).sum();
        data.put("unXunTotal", unXunTotal);
        // 本月已巡更点总数
        int actualXunTotal = result.stream().mapToInt(a -> Convert.toInt(a.getActualXunTotal())).sum();
        data.put("actualXunTotal", actualXunTotal);
        // 本月未巡更点总数
        int weiXun = result.stream().filter(a -> DateUtil.compare(new Date(), DateUtil.parseDate(a.getStaticDate())) < 0).mapToInt(XunStaticMonth::getPlanLocationTotal).sum();
        data.put("weiXun", weiXun);
        // 本月漏巡率
        int xunTotal = result.stream().mapToInt(a -> Convert.toInt(a.getXunTotal())).sum();
        double unPercent = (xunTotal == 0) ? (double) 0 : NumberUtil.mul(NumberUtil.div(unXunTotal, xunTotal, 4), 100);
        data.put("unPercent", unPercent);
        return data;
    }


}
