package com.ruoyi.qdtl.service.impl;

import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.qdtl.domain.TlArea;
import com.ruoyi.qdtl.mapper.TlAreaMapper;
import com.ruoyi.qdtl.service.ITlAreaService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 区域管理Service业务层处理
 *
 * @author ruoyi
 * @date 2022-02-21
 */
@Service
public class TlAreaServiceImpl implements ITlAreaService {
    @Autowired
    private TlAreaMapper tlAreaMapper;

    /**
     * 查询区域管理
     *
     * @param id 区域管理主键
     * @return 区域管理
     */
    @Override
    public TlArea selectTlAreaById(Long id) {
        return tlAreaMapper.selectTlAreaById(id);
    }

    /**
     * 查询区域管理列表
     *
     * @param tlArea 区域管理
     * @return 区域管理
     */
    @Override
    public List<TlArea> selectTlAreaList(TlArea tlArea) {
        return tlAreaMapper.selectTlAreaList(tlArea);
    }

    /**
     * 新增区域管理
     *
     * @param tlArea 区域管理
     * @return 结果
     */
    @Override
    public int insertTlArea(TlArea tlArea) {
        tlArea.setCreateTime(DateUtils.getNowDate());
        // 控制编号和名称不能重复
        int count = tlAreaMapper.selectExist(tlArea);
        if (count > 0) {
            throw new ServiceException("编号或名称已存在，请更换");
        }
        return tlAreaMapper.insertTlArea(tlArea);
    }

    /**
     * 修改区域管理
     *
     * @param tlArea 区域管理
     * @return 结果
     */
    @Override
    public int updateTlArea(TlArea tlArea) {
        tlArea.setUpdateTime(DateUtils.getNowDate());
        // 查询原有的数据
        TlArea old = selectTlAreaById(tlArea.getId());
        if (old != null) {
            if (!old.getAreaName().equals(tlArea.getAreaName())) {
                // 校验是否重复
                TlArea check = new TlArea();
                check.setAreaName(tlArea.getAreaName());
                List<TlArea> tlAreas = tlAreaMapper.selectTlAreaList(check);
                if (tlAreas.size() > 0) {
                    throw new ServiceException("名称已存在，请更换");
                }
            }
        }
        return tlAreaMapper.updateTlArea(tlArea);
    }

    /**
     * 批量删除区域管理
     *
     * @param ids 需要删除的区域管理主键
     * @return 结果
     */
    @Override
    public int deleteTlAreaByIds(Long[] ids) {
        return tlAreaMapper.deleteTlAreaByIds(ids);
    }

    /**
     * 删除区域管理信息
     *
     * @param id 区域管理主键
     * @return 结果
     */
    @Override
    public int deleteTlAreaById(Long id) {
        return tlAreaMapper.deleteTlAreaById(id);
    }

    @Override
    public List<TlArea> selectAreaList() {
        return tlAreaMapper.selectAreaList();
    }
}
