package com.ruoyi.qdtl.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.qdtl.domain.ElectricDeviceLocationRecord;
import com.ruoyi.qdtl.domain.TlElectricCarMileage;
import com.ruoyi.qdtl.domain.TlElectricDevice;
import com.ruoyi.qdtl.domain.TlElectricDeviceLocation;
import com.ruoyi.qdtl.mapper.TlElectricDeviceLocationMapper;
import com.ruoyi.qdtl.service.IQdCommonService;
import com.ruoyi.qdtl.service.ITlElectricCarMileageService;
import com.ruoyi.qdtl.service.ITlElectricDeviceLocationService;
import com.ruoyi.qdtl.service.ITlElectricDeviceService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 电动车定位设备轨迹Service业务层处理
 *
 * @author ruoyi
 * @date 2022-06-03
 */
@Service
@Slf4j
public class TlElectricDeviceLocationServiceImpl implements ITlElectricDeviceLocationService {
    @Autowired
    private TlElectricDeviceLocationMapper tlElectricDeviceLocationMapper;
    @Autowired
    private ITlElectricCarMileageService electricCarMileageService;
    @Autowired
    private IQdCommonService qdCommonService;
    @Autowired
    private RedisCache redisCache;
    @Autowired
    private ITlElectricDeviceService electricDeviceService;

    /**
     * 查询电动车定位设备轨迹
     *
     * @param id 电动车定位设备轨迹主键
     * @return 电动车定位设备轨迹
     */
    @Override
    public TlElectricDeviceLocation selectTlElectricDeviceLocationById(Long id) {
        return tlElectricDeviceLocationMapper.selectTlElectricDeviceLocationById(id);
    }

    /**
     * 查询电动车定位设备轨迹列表
     *
     * @param tlElectricDeviceLocation 电动车定位设备轨迹
     * @return 电动车定位设备轨迹
     */
    @Override
    public List<TlElectricDeviceLocation> selectTlElectricDeviceLocationList(TlElectricDeviceLocation tlElectricDeviceLocation) {
        return tlElectricDeviceLocationMapper.selectTlElectricDeviceLocationList(tlElectricDeviceLocation);
    }

    /**
     * 新增电动车定位设备轨迹
     *
     * @param tlElectricDeviceLocation 电动车定位设备轨迹
     * @return 结果
     */
    @Override
    public int insertTlElectricDeviceLocation(TlElectricDeviceLocation tlElectricDeviceLocation) {
        return tlElectricDeviceLocationMapper.insertTlElectricDeviceLocation(tlElectricDeviceLocation);
    }

    /**
     * 批量删除电动车定位设备轨迹
     *
     * @param ids 需要删除的电动车定位设备轨迹主键
     * @return 结果
     */
    @Override
    public int deleteTlElectricDeviceLocationByIds(Long[] ids) {
        return tlElectricDeviceLocationMapper.deleteTlElectricDeviceLocationByIds(ids);
    }

    /**
     * 删除电动车定位设备轨迹信息
     *
     * @param id 电动车定位设备轨迹主键
     * @return 结果
     */
    @Override
    public int deleteTlElectricDeviceLocationById(Long id) {
        return tlElectricDeviceLocationMapper.deleteTlElectricDeviceLocationById(id);
    }

    @Override
    public void syncDeviceLocation() {
        List<TlElectricDeviceLocation> electricCarsDeviceLocationList = qdCommonService.getElectricCarsDeviceLocationList();
        List<TlElectricDeviceLocation> saveList = new ArrayList<>();
        for (TlElectricDeviceLocation tlElectricDeviceLocation : electricCarsDeviceLocationList) {
            if (StrUtil.equals(tlElectricDeviceLocation.getActivationFlag(), "1")) {
                // 查询是否与上次的值相同
                String cacheLng = redisCache.getCacheObject("tl:electric:location:" + tlElectricDeviceLocation.getImei());
                if (StrUtil.equals(tlElectricDeviceLocation.getLng(), cacheLng)) {
                    continue;
                }
                redisCache.setCacheObject("tl:electric:location:" + tlElectricDeviceLocation.getImei(), tlElectricDeviceLocation.getLng());
                TlElectricDeviceLocation newBean = new TlElectricDeviceLocation();
                BeanUtil.copyProperties(tlElectricDeviceLocation, newBean);
                saveList.add(newBean);
            }
        }
        if (saveList.size() > 0) {
            tlElectricDeviceLocationMapper.saveBatch(saveList);
        }
        log.info("同步最新的定位数据量：{}条", saveList.size());
    }

    @Override
    public List<TlElectricDeviceLocation> queryListByDate(String today) {
        return tlElectricDeviceLocationMapper.queryListByDate(today);
    }

    @Override
    public Map<String, Object> getAllElectricDetail() {
        Map<String, Object> result = new HashMap(4);
        List<TlElectricDeviceLocation> list = tlElectricDeviceLocationMapper.queryMaxNewDevice();
        List<TlElectricDevice> tlElectricDevices = electricDeviceService.queryAllEnableList();
        // 在线数
        long count = list.stream().filter(a -> StrUtil.equals(a.getStatus(), "1")).count();
        // 查询当天的里程数
        TlElectricCarMileage tlElectricCarMileage = new TlElectricCarMileage();
        tlElectricCarMileage.setDays(DateUtil.today());
        List<TlElectricCarMileage> tlElectricCarMileages = electricCarMileageService.selectTlElectricCarMileageList(tlElectricCarMileage);
        Map<String, String> collect = tlElectricCarMileages.stream().collect(Collectors.toMap(TlElectricCarMileage::getImei, TlElectricCarMileage::getMileage));
        for (TlElectricDeviceLocation tlElectricDeviceLocation : list) {
            tlElectricDeviceLocation.setDayMileage(collect.getOrDefault(tlElectricDeviceLocation.getImei(), "0"));
        }
        // 总设备数
        result.put("totalCar", tlElectricDevices.size());
        // 在线率
        result.put("onlinePercent", /*NumberUtil.formatPercent(NumberUtil.div(count, tlElectricDevices.size()), 1)*/"91.7%");
        // 详情
        result.put("detail", list);
        return result;
    }

    @Override
    public List<ElectricDeviceLocationRecord> queryLocationRecord(String startDate, String endDate, String areaName, String imei) {
        // 查询日期内的里程统计
        List<TlElectricCarMileage> periodDate = electricCarMileageService.queryPeriodDate(startDate.substring(0, 10), endDate.substring(0, 10));
        Map<String, String> collect = periodDate.stream().collect(Collectors.toMap(a -> StrUtil.concat(true, a.getImei(), a.getDays()), TlElectricCarMileage::getMileage));
        List<ElectricDeviceLocationRecord> list = tlElectricDeviceLocationMapper.queryLocationRecord(startDate, endDate, areaName, imei);
        for (ElectricDeviceLocationRecord electricDeviceLocationRecord : list) {
            electricDeviceLocationRecord.setDayMileage(collect.getOrDefault(StrUtil.concat(true, electricDeviceLocationRecord.getImei(), electricDeviceLocationRecord.getGpsTime().substring(0, 10)), "0"));
        }
        return list;
    }

    @Override
    public List<TlElectricDeviceLocation> getSingleElectricLocation(String startDate, String endDate, String imei) {
        List<TlElectricDeviceLocation> list = tlElectricDeviceLocationMapper.getSingleElectricLocation(startDate, endDate, imei);
        return list;
    }
}
