package com.ruoyi.qdtl.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.StrUtil;
import com.ruoyi.qdtl.domain.ElectricCarArea;
import com.ruoyi.qdtl.domain.TlArea;
import com.ruoyi.qdtl.domain.TlElectricDevice;
import com.ruoyi.qdtl.domain.TlElectricDeviceLocation;
import com.ruoyi.qdtl.mapper.TlElectricDeviceLocationMapper;
import com.ruoyi.qdtl.mapper.TlElectricDeviceMapper;
import com.ruoyi.qdtl.service.IQdCommonService;
import com.ruoyi.qdtl.service.ITlAreaService;
import com.ruoyi.qdtl.service.ITlElectricDeviceLocationService;
import com.ruoyi.qdtl.service.ITlElectricDeviceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 电动车定位设备Service业务层处理
 *
 * @author ruoyi
 * @date 2022-06-03
 */
@Service
public class TlElectricDeviceServiceImpl implements ITlElectricDeviceService {
    @Autowired
    private TlElectricDeviceMapper tlElectricDeviceMapper;
    @Autowired
    private IQdCommonService qdCommonService;
    @Autowired
    private ITlAreaService areaService;
    @Autowired
    private ITlElectricDeviceLocationService electricDeviceLocationService;
    @Autowired
    private TlElectricDeviceLocationMapper tlElectricDeviceLocationMapper;

    /**
     * 查询电动车定位设备
     *
     * @param imei 电动车定位设备主键
     * @return 电动车定位设备
     */
    @Override
    public TlElectricDevice selectTlElectricDeviceByImei(String imei) {
        return tlElectricDeviceMapper.selectTlElectricDeviceByImei(imei);
    }

    /**
     * 查询电动车定位设备列表
     *
     * @param tlElectricDevice 电动车定位设备
     * @return 电动车定位设备
     */
    @Override
    public List<TlElectricDevice> selectTlElectricDeviceList(TlElectricDevice tlElectricDevice) {
        return tlElectricDeviceMapper.selectTlElectricDeviceList(tlElectricDevice);
    }

    /**
     * 新增电动车定位设备
     *
     * @param tlElectricDevice 电动车定位设备
     * @return 结果
     */
    @Override
    public int insertTlElectricDevice(TlElectricDevice tlElectricDevice) {
        return tlElectricDeviceMapper.insertTlElectricDevice(tlElectricDevice);
    }

    /**
     * 修改电动车定位设备
     *
     * @param tlElectricDevice 电动车定位设备
     * @return 结果
     */
    @Override
    public int updateTlElectricDevice(TlElectricDevice tlElectricDevice) {
        return tlElectricDeviceMapper.updateTlElectricDevice(tlElectricDevice);
    }

    /**
     * 批量删除电动车定位设备
     *
     * @param imeis 需要删除的电动车定位设备主键
     * @return 结果
     */
    @Override
    public int deleteTlElectricDeviceByImeis(String[] imeis) {
        return tlElectricDeviceMapper.deleteTlElectricDeviceByImeis(imeis);
    }

    /**
     * 删除电动车定位设备信息
     *
     * @param imei 电动车定位设备主键
     * @return 结果
     */
    @Override
    public int deleteTlElectricDeviceByImei(String imei) {
        return tlElectricDeviceMapper.deleteTlElectricDeviceByImei(imei);
    }

    @Override
    @Transactional
    public String syncDevice() {
        int delSum = 0, addSum = 0, updateSum = 0;
        List<TlElectricDevice> remoteList = qdCommonService.getElectricCarsDeviceList();
        List<String> remoteImei = remoteList.stream().map(TlElectricDevice::getImei).collect(Collectors.toList());
        // 查询本地的数据，若存在差集则删除
        List<TlElectricDevice> localList = this.selectTlElectricDeviceList(new TlElectricDevice());
        List<String> localImei = localList.stream().map(TlElectricDevice::getImei).collect(Collectors.toList());
        // 待删除的数据
        List<String> removeImei = CollUtil.subtractToList(localImei, remoteImei);
        // 待新增的数据
        List<String> saveImei = CollUtil.subtractToList(remoteImei, localImei);
        // 待更新的数据
        Collection<String> intersection = CollUtil.intersection(localImei, remoteImei);

        // 删除
        if (removeImei.size() > 0) {
            deleteTlElectricDeviceByImeis(Convert.toStrArray(removeImei));
            delSum = removeImei.size();
        }
        for (TlElectricDevice tlElectricDevice : remoteList) {
            if (saveImei.contains(tlElectricDevice.getImei())) {
                insertTlElectricDevice(tlElectricDevice);
                addSum++;
            } else if (intersection.contains(tlElectricDevice.getImei())) {
                updateTlElectricDevice(tlElectricDevice);
                updateSum++;
            }
        }
        return StrUtil.concat(true, "此次同步共删除数量", Convert.toStr(delSum),
                "条；共新增数量", Convert.toStr(addSum),
                "条；共更新数量", Convert.toStr(updateSum), "条。");
    }

    @Override
    public List<TlElectricDevice> queryAllEnableList() {
        return tlElectricDeviceMapper.queryAllEnableList();
    }

    @Override
    public List<ElectricCarArea> queryCarStaticByArea() {
        List<ElectricCarArea> result = new ArrayList<>();
        TlArea tlArea = new TlArea();
        tlArea.setAreaType("1");
        List<TlArea> areas = areaService.selectTlAreaList(tlArea);
        List<TlElectricDeviceLocation> list = tlElectricDeviceLocationMapper.queryMaxNewDevice();
        // 在线数
        Map<String, List<TlElectricDeviceLocation>> onlineCollect = list.stream().filter(a -> StrUtil.equals(a.getStatus(), "1")).collect(Collectors.groupingBy(a -> StrUtil.sub(a.getDeviceName(), 0, 3)));
        // 设备数量
        List<TlElectricDevice> tlElectricDevices = this.queryAllEnableList();
        Map<String, List<TlElectricDevice>> collect = tlElectricDevices.stream().collect(Collectors.groupingBy(a -> StrUtil.sub(a.getDeviceName(), 0, 3)));
        for (TlArea area : areas) {
            ElectricCarArea electricCarArea = new ElectricCarArea();
            electricCarArea.setAreaId(area.getId());
            electricCarArea.setAreaName(area.getAreaName());
            // 查询每个区域总数
            electricCarArea.setTotalCar(collect.get(area.getAreaName()) == null ? "0" : Convert.toStr(collect.get(area.getAreaName()).size()));
            electricCarArea.setOnlineCar(onlineCollect.get(area.getAreaName()) == null ? "0" : Convert.toStr(onlineCollect.get(area.getAreaName()).size()));
            result.add(electricCarArea);
        }
        return result;
    }
}
