package com.ruoyi.qdtl.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.qdtl.domain.TlInspectionDevice;
import com.ruoyi.qdtl.mapper.TlInspectionDeviceMapper;
import com.ruoyi.qdtl.service.ITlInspectionDeviceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 巡检设备管理Service业务层处理
 *
 * @author ruoyi
 * @date 2022-02-22
 */
@Service
public class TlInspectionDeviceServiceImpl implements ITlInspectionDeviceService {
    @Autowired
    private TlInspectionDeviceMapper tlInspectionDeviceMapper;
    @Autowired
    private RedisCache redisCache;

    /**
     * 查询巡检设备管理
     *
     * @param id 巡检设备管理主键
     * @return 巡检设备管理
     */
    @Override
    public TlInspectionDevice selectTlInspectionDeviceById(Long id) {
        TlInspectionDevice cacheObject = redisCache.getCacheObject("tl:device:info:" + id);
        if (cacheObject != null) {
            return cacheObject;
        }
        TlInspectionDevice tlInspectionDevice = tlInspectionDeviceMapper.selectTlInspectionDeviceById(id);
        redisCache.setCacheObject("tl:device:info:" + id, tlInspectionDevice);
        return tlInspectionDevice;
    }

    /**
     * 查询巡检设备管理列表
     *
     * @param tlInspectionDevice 巡检设备管理
     * @return 巡检设备管理
     */
    @Override
    public List<TlInspectionDevice> selectTlInspectionDeviceList(TlInspectionDevice tlInspectionDevice) {
        return tlInspectionDeviceMapper.selectTlInspectionDeviceList(tlInspectionDevice);
    }

    /**
     * 新增巡检设备管理
     *
     * @param tlInspectionDevice 巡检设备管理
     * @return 结果
     */
    @Override
    public int insertTlInspectionDevice(TlInspectionDevice tlInspectionDevice) {
        tlInspectionDevice.setCreateTime(DateUtils.getNowDate());
        // 控制编号和名称不能重复
        int count = tlInspectionDeviceMapper.selectExist(tlInspectionDevice);
        if (count > 0) {
            throw new ServiceException("编号或名称已存在，请更换");
        }
        return tlInspectionDeviceMapper.insertTlInspectionDevice(tlInspectionDevice);
    }

    /**
     * 修改巡检设备管理
     *
     * @param tlInspectionDevice 巡检设备管理
     * @return 结果
     */
    @Override
    public int updateTlInspectionDevice(TlInspectionDevice tlInspectionDevice) {
        tlInspectionDevice.setUpdateTime(DateUtils.getNowDate());
        // 查询原有的数据
        TlInspectionDevice old = selectTlInspectionDeviceById(tlInspectionDevice.getId());
        if (old != null) {
            if (!old.getDeviceCode().equals(tlInspectionDevice.getDeviceCode())) {
                // 校验是否重复
                TlInspectionDevice check = new TlInspectionDevice();
                check.setDeviceCode(tlInspectionDevice.getDeviceCode());
                List<TlInspectionDevice> tlInspectionDevices = tlInspectionDeviceMapper.selectTlInspectionDeviceList(check);
                if (tlInspectionDevices.size() > 0) {
                    throw new ServiceException("编号已存在，请更换");
                }
            }
            if (!old.getDeviceName().equals(tlInspectionDevice.getDeviceName())) {
                // 校验是否重复
                TlInspectionDevice check = new TlInspectionDevice();
                check.setDeviceName(tlInspectionDevice.getDeviceName());
                List<TlInspectionDevice> tlInspectionDevices = tlInspectionDeviceMapper.selectTlInspectionDeviceList(check);
                if (tlInspectionDevices.size() > 0) {
                    throw new ServiceException("名称已存在，请更换");
                }
            }
        }
        int i = tlInspectionDeviceMapper.updateTlInspectionDevice(tlInspectionDevice);
        redisCache.deleteObject("tl:device:info:" + tlInspectionDevice.getId());
        return i;
    }

    /**
     * 批量删除巡检设备管理
     *
     * @param ids 需要删除的巡检设备管理主键
     * @return 结果
     */
    @Override
    public int deleteTlInspectionDeviceByIds(Long[] ids) {
        return tlInspectionDeviceMapper.deleteTlInspectionDeviceByIds(ids);
    }

    /**
     * 删除巡检设备管理信息
     *
     * @param id 巡检设备管理主键
     * @return 结果
     */
    @Override
    public int deleteTlInspectionDeviceById(Long id) {
        redisCache.deleteObject("tl:device:info:" + id);
        return tlInspectionDeviceMapper.deleteTlInspectionDeviceById(id);
    }

    @Override
    public Map<String, Integer> queryDeviceByArea() {
        List<JSONObject> jsonObjects = tlInspectionDeviceMapper.queryDeviceByArea();
        return jsonObjects.stream().collect(Collectors.toMap(a -> a.getString("areaId"), b -> b.getInteger("cnt")));
    }

    @Override
    public TlInspectionDevice queryByThirdId(String thirdId) {
        return tlInspectionDeviceMapper.queryByThirdId(thirdId);
    }

    @Override
    public void deleteLocationByVersion(int version) {
        tlInspectionDeviceMapper.deleteLocationByVersion(version);
    }

    @Override
    public int queryMaxVsersion() {
        return tlInspectionDeviceMapper.queryMaxVsersion();
    }
}
