package com.ruoyi.qdtl.service.impl;

import java.util.List;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.qdtl.domain.TlInspectionLineLocation;
import com.ruoyi.qdtl.service.ITlInspectionLineLocationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.qdtl.mapper.TlInspectionLineMapper;
import com.ruoyi.qdtl.domain.TlInspectionLine;
import com.ruoyi.qdtl.service.ITlInspectionLineService;

/**
 * 巡检线路管理Service业务层处理
 * 
 * @author ruoyi
 * @date 2022-03-03
 */
@Service
public class TlInspectionLineServiceImpl implements ITlInspectionLineService 
{
    @Autowired
    private TlInspectionLineMapper tlInspectionLineMapper;
    @Autowired
    private ITlInspectionLineLocationService tlInspectionLineLocationService;

    /**
     * 查询巡检线路管理
     * 
     * @param id 巡检线路管理主键
     * @return 巡检线路管理
     */
    @Override
    public TlInspectionLine selectTlInspectionLineById(Long id)
    {
        return tlInspectionLineMapper.selectTlInspectionLineById(id);
    }

    /**
     * 查询巡检线路管理列表
     * 
     * @param tlInspectionLine 巡检线路管理
     * @return 巡检线路管理
     */
    @Override
    public List<TlInspectionLine> selectTlInspectionLineList(TlInspectionLine tlInspectionLine)
    {
        return tlInspectionLineMapper.selectTlInspectionLineList(tlInspectionLine);
    }

    /**
     * 新增巡检线路管理
     * 
     * @param tlInspectionLine 巡检线路管理
     * @return 结果
     */
    @Override
    public int insertTlInspectionLine(TlInspectionLine tlInspectionLine)
    {
        tlInspectionLine.setCreateTime(DateUtils.getNowDate());
        int row = tlInspectionLineMapper.insertTlInspectionLine(tlInspectionLine);
        for (TlInspectionLineLocation lineLocation : tlInspectionLine.getLineLocations()) {
            lineLocation.setLineId(tlInspectionLine.getId());
        }
        // 新增路线中检测点位数据
        if (tlInspectionLine.getLineLocations().size() > 0) {
            tlInspectionLineLocationService.batchInsert(tlInspectionLine.getLineLocations());
        }
        return row;
    }

    /**
     * 修改巡检线路管理
     * 
     * @param tlInspectionLine 巡检线路管理
     * @return 结果
     */
    @Override
    public int updateTlInspectionLine(TlInspectionLine tlInspectionLine)
    {
        tlInspectionLine.setUpdateTime(DateUtils.getNowDate());
        int row = tlInspectionLineMapper.updateTlInspectionLine(tlInspectionLine);
        // 删除关系后再新增
        tlInspectionLineLocationService.deleteLineLocation(tlInspectionLine.getId());
        for (TlInspectionLineLocation lineLocation : tlInspectionLine.getLineLocations()) {
            lineLocation.setLineId(tlInspectionLine.getId());
        }
        // 新增路线中检测点位数据
        if (tlInspectionLine.getLineLocations().size() > 0) {
            tlInspectionLineLocationService.batchInsert(tlInspectionLine.getLineLocations());
        }
        return row;
    }

    /**
     * 批量删除巡检线路管理
     * 
     * @param ids 需要删除的巡检线路管理主键
     * @return 结果
     */
    @Override
    public int deleteTlInspectionLineByIds(Long[] ids)
    {
        return tlInspectionLineMapper.deleteTlInspectionLineByIds(ids);
    }

    /**
     * 删除巡检线路管理信息
     * 
     * @param id 巡检线路管理主键
     * @return 结果
     */
    @Override
    public int deleteTlInspectionLineById(Long id)
    {
        return tlInspectionLineMapper.deleteTlInspectionLineById(id);
    }
}
