package com.ruoyi.qdtl.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.qdtl.domain.TlInspectionLocation;
import com.ruoyi.qdtl.mapper.TlInspectionLocationMapper;
import com.ruoyi.qdtl.service.ITlInspectionLocationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 巡检点管理Service业务层处理
 *
 * @author ruoyi
 * @date 2022-02-22
 */
@Service
public class TlInspectionLocationServiceImpl implements ITlInspectionLocationService {
    @Autowired
    private TlInspectionLocationMapper tlInspectionLocationMapper;

    /**
     * 查询巡检点管理
     *
     * @param id 巡检点管理主键
     * @return 巡检点管理
     */
    @Override
    public TlInspectionLocation selectTlInspectionLocationById(Long id) {
        return tlInspectionLocationMapper.selectTlInspectionLocationById(id);
    }

    /**
     * 查询巡检点管理列表
     *
     * @param tlInspectionLocation 巡检点管理
     * @return 巡检点管理
     */
    @Override
    public List<TlInspectionLocation> selectTlInspectionLocationList(TlInspectionLocation tlInspectionLocation) {
        return tlInspectionLocationMapper.selectTlInspectionLocationList(tlInspectionLocation);
    }

    /**
     * 新增巡检点管理
     *
     * @param tlInspectionLocation 巡检点管理
     * @return 结果
     */
    @Override
    public int insertTlInspectionLocation(TlInspectionLocation tlInspectionLocation) {
        tlInspectionLocation.setCreateTime(DateUtils.getNowDate());
        // 控制编号和名称不能重复
        /*int count = tlInspectionLocationMapper.selectExist(tlInspectionLocation);
        if (count > 0) {
            throw new ServiceException("编号或名称已存在，请更换");
        }*/
        return tlInspectionLocationMapper.insertTlInspectionLocation(tlInspectionLocation);
    }

    /**
     * 修改巡检点管理
     *
     * @param tlInspectionLocation 巡检点管理
     * @return 结果
     */
    @Override
    public int updateTlInspectionLocation(TlInspectionLocation tlInspectionLocation) {
        tlInspectionLocation.setUpdateTime(DateUtils.getNowDate());
        // 查询原有的数据
        /*TlInspectionLocation old = selectTlInspectionLocationById(tlInspectionLocation.getId());
        if (old != null) {
            if (!old.getLocationCode().equals(tlInspectionLocation.getLocationCode())) {
                // 校验是否重复
                TlInspectionLocation check = new TlInspectionLocation();
                check.setLocationCode(tlInspectionLocation.getLocationCode());
                List<TlInspectionLocation> tlInspectionLocations = tlInspectionLocationMapper.selectTlInspectionLocationList(check);
                if (tlInspectionLocations.size() > 0) {
                    throw new ServiceException("编号已存在，请更换");
                }
            }
            if (!old.getLocationName().equals(tlInspectionLocation.getLocationName())) {
                // 校验是否重复
                TlInspectionLocation check = new TlInspectionLocation();
                check.setLocationName(tlInspectionLocation.getLocationName());
                List<TlInspectionLocation> tlInspectionLocations = tlInspectionLocationMapper.selectTlInspectionLocationList(check);
                if (tlInspectionLocations.size() > 0) {
                    throw new ServiceException("名称已存在，请更换");
                }
            }
        }*/
        return tlInspectionLocationMapper.updateTlInspectionLocation(tlInspectionLocation);
    }

    /**
     * 批量删除巡检点管理
     *
     * @param ids 需要删除的巡检点管理主键
     * @return 结果
     */
    @Override
    public int deleteTlInspectionLocationByIds(Long[] ids) {
        return tlInspectionLocationMapper.deleteTlInspectionLocationByIds(ids);
    }

    /**
     * 删除巡检点管理信息
     *
     * @param id 巡检点管理主键
     * @return 结果
     */
    @Override
    public int deleteTlInspectionLocationById(Long id) {
        return tlInspectionLocationMapper.deleteTlInspectionLocationById(id);
    }

    @Override
    public Map<String, Integer> queryLocationByArea() {
        List<JSONObject> jsonObjects = tlInspectionLocationMapper.queryLocationByArea();
        return jsonObjects.stream().collect(Collectors.toMap(a -> a.getString("areaId"), b -> b.getInteger("cnt")));
    }

    @Override
    public List<TlInspectionLocation> queryLocationByLineIds(List<Long> idCollect) {
        return tlInspectionLocationMapper.queryLocationByLineIds(idCollect);
    }

    @Override
    public List<TlInspectionLocation> selectLocationList(TlInspectionLocation tlInspectionLocation) {
        return tlInspectionLocationMapper.selectLocationList(tlInspectionLocation);
    }

    @Override
    public TlInspectionLocation queryByThirdId(String thirdId) {
        return tlInspectionLocationMapper.queryByThirdId(thirdId);
    }

    @Override
    public int queryMaxVsersion() {
        return tlInspectionLocationMapper.queryMaxVsersion();
    }

    @Override
    public void deleteLocationByVersion(int version) {
        tlInspectionLocationMapper.deleteLocationByVersion(version);
    }

    /**
     * 获取线路的点位
     *
     * @param lineId
     * @return
     */
    @Override
    public List<TlInspectionLocation> queryLocationByLineId(Long lineId) {
        return tlInspectionLocationMapper.queryLocationByLineId(lineId);
    }

    @Override
    public List<String> queryLocationByLocationIds(Collection<Long> locations) {
        return tlInspectionLocationMapper.queryLocationByLocationIds(locations);
    }
}
