package com.ruoyi.qdtl.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.ruoyi.common.core.text.Convert;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.qdtl.domain.PlanSchedule;
import com.ruoyi.qdtl.domain.PlanUser;
import com.ruoyi.qdtl.domain.TlInspectionLocation;
import com.ruoyi.qdtl.domain.TlInspectionPlan;
import com.ruoyi.qdtl.domain.TlInspectionPlanUser;
import com.ruoyi.qdtl.mapper.TlInspectionPlanMapper;
import com.ruoyi.qdtl.service.ITlInspectionLineLocationService;
import com.ruoyi.qdtl.service.ITlInspectionLocationService;
import com.ruoyi.qdtl.service.ITlInspectionPlanService;
import com.ruoyi.qdtl.service.ITlInspectionPlanUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 巡检计划管理Service业务层处理
 *
 * @author ruoyi
 * @date 2022-03-03
 */
@Service
public class TlInspectionPlanServiceImpl implements ITlInspectionPlanService {
    @Autowired
    private TlInspectionPlanMapper tlInspectionPlanMapper;
    @Autowired
    private ITlInspectionPlanUserService tlInspectionPlanUserService;
    @Autowired
    private ITlInspectionLineLocationService tlInspectionLineLocationService;
    @Autowired
    private ITlInspectionLocationService tlInspectionLocationService;

    /**
     * 查询巡检计划管理
     *
     * @param id 巡检计划管理主键
     * @return 巡检计划管理
     */
    @Override
    public TlInspectionPlan selectTlInspectionPlanById(Long id) {
        return tlInspectionPlanMapper.selectTlInspectionPlanById(id);
    }

    /**
     * 查询巡检计划管理列表
     *
     * @param tlInspectionPlan 巡检计划管理
     * @return 巡检计划管理
     */
    @Override
    public List<TlInspectionPlan> selectTlInspectionPlanList(TlInspectionPlan tlInspectionPlan) {
        return tlInspectionPlanMapper.selectTlInspectionPlanList(tlInspectionPlan);
    }

    /**
     * 新增巡检计划管理
     *
     * @param tlInspectionPlan 巡检计划管理
     * @return 结果
     */
    @Override
    @Transactional
    public int insertTlInspectionPlan(TlInspectionPlan tlInspectionPlan) {
        tlInspectionPlan.setCreateTime(DateUtils.getNowDate());
        int row = tlInspectionPlanMapper.insertTlInspectionPlan(tlInspectionPlan);
        // 校验人员是否存在重复的计划范围
        checkPlanUser(tlInspectionPlan, tlInspectionPlan.getId());
        for (TlInspectionPlanUser planUser : tlInspectionPlan.getPlanUsers()) {
            planUser.setPlanId(tlInspectionPlan.getId());
        }
        if (tlInspectionPlan.getPlanUsers().size() > 0) {
            tlInspectionPlanUserService.batchInsert(tlInspectionPlan.getPlanUsers());
        }
        return row;
    }

    /**
     * 修改巡检计划管理
     *
     * @param tlInspectionPlan 巡检计划管理
     * @return 结果
     */
    @Override
    public int updateTlInspectionPlan(TlInspectionPlan tlInspectionPlan) {
        tlInspectionPlan.setUpdateTime(DateUtils.getNowDate());
        int row = tlInspectionPlanMapper.updateTlInspectionPlan(tlInspectionPlan);
        // 校验人员是否存在重复的计划范围
        checkPlanUser(tlInspectionPlan, tlInspectionPlan.getId());
        // 删除关系后再新增
        tlInspectionPlanUserService.deletePlanUser(tlInspectionPlan.getId());
        for (TlInspectionPlanUser planUser : tlInspectionPlan.getPlanUsers()) {
            planUser.setPlanId(tlInspectionPlan.getId());
        }
        if (tlInspectionPlan.getPlanUsers().size() > 0) {
            tlInspectionPlanUserService.batchInsert(tlInspectionPlan.getPlanUsers());
        }
        return row;
    }

    /**
     * 批量删除巡检计划管理
     *
     * @param ids 需要删除的巡检计划管理主键
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteTlInspectionPlanByIds(Long[] ids) {
        int i = tlInspectionPlanMapper.deleteTlInspectionPlanByIds(ids);
        // 删除相关人员
        tlInspectionPlanUserService.deleteTlInspectionPlanUserByPlanIds(ids);
        return i;
    }

    /**
     * 删除巡检计划管理信息
     *
     * @param id 巡检计划管理主键
     * @return 结果
     */
    @Override
    public int deleteTlInspectionPlanById(Long id) {
        return tlInspectionPlanMapper.deleteTlInspectionPlanById(id);
    }

    @Override
    public List<TlInspectionPlan> queryPlanByDate(String queryDate) {
        return tlInspectionPlanMapper.queryPlanByDate(queryDate);
    }

    @Override
    public List<PlanUser> queryPlanUser(Long id) {
        return tlInspectionPlanMapper.queryPlanUser(id);
    }

    /**
     * 获取排班表
     *
     * @param nickName
     * @param startDate
     * @param endDate
     * @return
     */
    @Override
    public List<PlanSchedule> getSchedule(String nickName, String startDate, String endDate) {
        // 如果startDate或者endDate为空，则默认展示最近7天的
        if (StrUtil.hasBlank(startDate, endDate)) {
            startDate = DateUtil.today();
            endDate = DateUtil.formatDate(DateUtil.offsetDay(new Date(), 6));
        }
        List<PlanSchedule> result = new ArrayList<>();
        List<TlInspectionPlan> list = tlInspectionPlanMapper.queryByDatePeriod(startDate, endDate);
        // 解析list
        for (TlInspectionPlan inspectionPlan : list) {
            PlanSchedule planSchedule = new PlanSchedule();
            List<TlInspectionLocation> lineLocations = tlInspectionLocationService.queryLocationByLineId(inspectionPlan.getLineId());
            planSchedule.setEffectDate(inspectionPlan.getStartDate() + "~" + inspectionPlan.getEndDate());
            planSchedule.setLocations(lineLocations.stream().map(TlInspectionLocation::getLocationName).collect(Collectors.toList()));
            planSchedule.setTimeUnit(StrUtil.split(inspectionPlan.getTimeUnit(), ","));
            // 确定当前时间范围内的人员
            List<DateTime> dateTimes = DateUtil.rangeToList(DateUtil.parseDate(startDate), DateUtil.parseDate(endDate), DateField.DAY_OF_YEAR);
            List<PlanSchedule.PlanDetail> planDetails = new ArrayList<>();
            // 查询计划的人员
            TlInspectionPlanUser tlInspectionPlanUser = new TlInspectionPlanUser();
            tlInspectionPlanUser.setPlanId(inspectionPlan.getId());
            if (StrUtil.isNotBlank(nickName)) {
                tlInspectionPlanUser.setNickName(nickName);
            }
            List<TlInspectionPlanUser> users = tlInspectionPlanUserService.selectTlInspectionPlanUserList(tlInspectionPlanUser);
            String userText = users.stream().map(TlInspectionPlanUser::getNickName).collect(Collectors.joining("、"));
            for (DateTime dateTime : dateTimes) {
                PlanSchedule.PlanDetail planDetail = new PlanSchedule.PlanDetail();
                planDetail.setQueryDate(DateUtil.formatDate(dateTime));
                int week = DateUtil.dayOfWeek(dateTime);
                planDetail.setWeek(week);
                if (CollUtil.contains(StrUtil.split(inspectionPlan.getWeeks(), ","), Convert.toStr(week))) {
                    planDetail.setNickName(userText);
                } else {
                    planDetail.setNickName("");
                }
                planDetails.add(planDetail);
            }
            planSchedule.setPlanDetails(planDetails);
            result.add(planSchedule);
        }
        return result;
    }

    /**
     * 校验计划中用户是否存在重复的
     *
     * @param tlInspectionPlan 待校验的计划数据
     * @param excludePlanId    排除的计划id
     */
    private void checkPlanUser(TlInspectionPlan tlInspectionPlan, Long excludePlanId) {
        // 此次的所有日期可能性
        List<String> currentDateTime = new ArrayList<>();
        List<DateTime> currentTimes = DateUtil.rangeToList(DateUtil.parseDate(tlInspectionPlan.getStartDate()), DateUtil.parseDate(tlInspectionPlan.getEndDate()), DateField.DAY_OF_YEAR);
        boolean currentFlag = !"1,2,3,4,5,6,7".equals(tlInspectionPlan.getWeeks());
        for (DateTime currentTime : currentTimes) {
            if (currentFlag) {
                int week = DateUtil.dayOfWeek(currentTime);
                if (!CollUtil.contains(StrUtil.split(tlInspectionPlan.getWeeks(), ","), Convert.toStr(week))) {
                    continue;
                }
            }
            List<String> split = StrUtil.split(tlInspectionPlan.getTimeUnit(), ",");
            for (String s : split) {
                for (String s1 : StrUtil.split(s, "~")) {
                    currentDateTime.add(DateUtil.formatDate(currentTime) + " " + s1);
                }
            }
        }
        // 当前计划所有的点位
        List<Long> currentLocation = tlInspectionLineLocationService.queryLocationId(tlInspectionPlan.getLineId());
        for (TlInspectionPlanUser planUser : tlInspectionPlan.getPlanUsers()) {
            String card = planUser.getCard();
            String userName = planUser.getNickName();
            // 查询该用户的计划
            List<TlInspectionPlan> list = tlInspectionPlanMapper.queryPlanUserExcludeId(card, excludePlanId);
            if (list != null && list.size() > 0) {
                for (TlInspectionPlan inspectionPlan : list) {
                    // 组装日期范围
                    // 已存在的
                    List<String> existDateTime = new ArrayList<>();
                    List<DateTime> existTimes = DateUtil.rangeToList(DateUtil.parseDate(inspectionPlan.getStartDate()), DateUtil.parseDate(inspectionPlan.getEndDate()), DateField.DAY_OF_YEAR);
                    boolean existFlag = !"1,2,3,4,5,6,7".equals(inspectionPlan.getWeeks());
                    for (DateTime existTime : existTimes) {
                        if (existFlag) {
                            int week = DateUtil.dayOfWeek(existTime);
                            if (!CollUtil.contains(StrUtil.split(inspectionPlan.getWeeks(), ","), Convert.toStr(week))) {
                                continue;
                            }
                        }
                        List<String> split = StrUtil.split(inspectionPlan.getTimeUnit(), ",");
                        for (String s : split) {
                            for (String s1 : StrUtil.split(s, "~")) {
                                existDateTime.add(DateUtil.formatDate(existTime) + " " + s1);
                            }
                        }
                    }
                    // 判断是否有重复的时间段
                    Collection<String> intersection = CollUtil.intersection(currentDateTime, existDateTime);
                    if (intersection.size() > 0) {
                        // 检查是否有相同的巡检点
                        List<Long> existLocation = tlInspectionLineLocationService.queryLocationId(inspectionPlan.getLineId());
                        Collection<Long> locations = CollUtil.intersection(currentLocation, existLocation);
                        if (locations.size() > 0) {
                            // 组装错误信息
                            List<String> locationNames = tlInspectionLocationService.queryLocationByLocationIds(locations);
                            String message = StrUtil.concat(true, userName, "在", inspectionPlan.getPlanName(), "中", String.join(",", intersection), "时间段内的点位：", String.join(",", locationNames), "存在相冲突的计划巡检点");
                            throw new ServiceException(message);
                        }
                    }
                }
            }
        }
    }

}
