package com.ruoyi.qdtl.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.qdtl.domain.AreaCheckInfoStatic;
import com.ruoyi.qdtl.domain.AreaCheckPercent;
import com.ruoyi.qdtl.domain.TlArea;
import com.ruoyi.qdtl.domain.TlPlanRecord;
import com.ruoyi.qdtl.mapper.TlPlanRecordMapper;
import com.ruoyi.qdtl.service.ITlAreaService;
import com.ruoyi.qdtl.service.ITlPlanRecordService;
import org.gavaghan.geodesy.Ellipsoid;
import org.gavaghan.geodesy.GeodeticCalculator;
import org.gavaghan.geodesy.GlobalCoordinates;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 巡检计划记录Service业务层处理
 *
 * @author ruoyi
 * @date 2022-03-30
 */
@Service
public class TlPlanRecordServiceImpl implements ITlPlanRecordService {
    @Autowired
    private TlPlanRecordMapper tlPlanRecordMapper;
    @Autowired
    private ITlAreaService tlAreaService;

    /**
     * 查询巡检计划记录
     *
     * @param id 巡检计划记录主键
     * @return 巡检计划记录
     */
    @Override
    public TlPlanRecord selectTlPlanRecordById(Long id) {
        return tlPlanRecordMapper.selectTlPlanRecordById(id);
    }

    /**
     * 查询巡检计划记录列表
     *
     * @param tlPlanRecord 巡检计划记录
     * @return 巡检计划记录
     */
    @Override
    public List<TlPlanRecord> selectTlPlanRecordList(TlPlanRecord tlPlanRecord) {
        return tlPlanRecordMapper.selectTlPlanRecordList(tlPlanRecord);
    }

    /**
     * 新增巡检计划记录
     *
     * @param tlPlanRecord 巡检计划记录
     * @return 结果
     */
    @Override
    public int insertTlPlanRecord(TlPlanRecord tlPlanRecord) {
        tlPlanRecord.setCreateTime(DateUtils.getNowDate());
        return tlPlanRecordMapper.insertTlPlanRecord(tlPlanRecord);
    }

    /**
     * 修改巡检计划记录
     *
     * @param tlPlanRecord 巡检计划记录
     * @return 结果
     */
    @Override
    public int updateTlPlanRecord(TlPlanRecord tlPlanRecord) {
        return tlPlanRecordMapper.updateTlPlanRecord(tlPlanRecord);
    }

    /**
     * 批量删除巡检计划记录
     *
     * @param ids 需要删除的巡检计划记录主键
     * @return 结果
     */
    @Override
    public int deleteTlPlanRecordByIds(Long[] ids) {
        return tlPlanRecordMapper.deleteTlPlanRecordByIds(ids);
    }

    /**
     * 删除巡检计划记录信息
     *
     * @param id 巡检计划记录主键
     * @return 结果
     */
    @Override
    public int deleteTlPlanRecordById(Long id) {
        return tlPlanRecordMapper.deleteTlPlanRecordById(id);
    }

    @Override
    public void batchInsert(List<TlPlanRecord> planRecords) {
        tlPlanRecordMapper.batchInsert(planRecords);
    }

    @Override
    public void updateRecordByDateAndCard(TlPlanRecord tlPlanRecord) {
        tlPlanRecordMapper.updateRecordByDateAndCard(tlPlanRecord);
    }

    /**
     * 查询巡查完成率
     *
     * @param date
     * @return
     */
    @Override
    public Map<String, Object> queryCheckPercent(String date) {
        Map<String, Object> mapResult = new HashMap<>(4);
        // 查询所有符合日期的数据
        List<TlPlanRecord> planRecords = tlPlanRecordMapper.selectTlPlanRecordListByDate(date);
        // 以各个镇分组
        Map<Long, List<TlPlanRecord>> areaCollect = planRecords.stream().filter(a -> ObjectUtil.isNotNull(a.getAreaId())).collect(Collectors.groupingBy(TlPlanRecord::getAreaId));
        // 再对每一个镇计算完成率
        List<AreaCheckPercent> result = new ArrayList<>();
        double totalCount = 0;
        double finishCount = 0;
        for (Map.Entry<Long, List<TlPlanRecord>> longListEntry : areaCollect.entrySet()) {
            AreaCheckPercent areaCheckPercent = new AreaCheckPercent();
            Long key = longListEntry.getKey();
            List<TlPlanRecord> value = longListEntry.getValue();
            // 对每一个镇的完成数据筛选出来
            List<TlPlanRecord> finish = value.stream().filter(a -> StrUtil.isNotBlank(a.getDeviceCode())).collect(Collectors.toList());
            // 统计总数，方便计算总的完成率
            totalCount = NumberUtil.add(totalCount, value.size());
            finishCount = NumberUtil.add(finishCount, finish.size());
            // 对完成数据中以巡检员卡号分组,以打卡时间排序，并计算总里程
            Map<String, List<TlPlanRecord>> cardCollect = finish.stream().collect(Collectors.groupingBy(TlPlanRecord::getCard));
            // 该区域的总里程
            double distance = 0;
            for (Map.Entry<String, List<TlPlanRecord>> stringListEntry : cardCollect.entrySet()) {
                // 时间排序
                List<TlPlanRecord> origin = stringListEntry.getValue().stream().sorted(Comparator.comparing(TlPlanRecord::getCreateTime)).collect(Collectors.toList());
                // 计算总里程
                if (origin.size() < 2) {
                    continue;
                }
                // 对原始list分成2组list，一个去掉头节点，一个去掉尾节点
                List<TlPlanRecord> subPre = CollUtil.sub(origin, 0, origin.size() - 1);
                List<TlPlanRecord> subAfter = CollUtil.sub(origin, 1, origin.size());
                for (int i = 0; i < subPre.size(); i++) {
                    String fencePre = subPre.get(i).getFence();
                    String fenceAfter = subAfter.get(i).getFence();
                    // 计算2点之间距离
                    GlobalCoordinates source = new GlobalCoordinates(Convert.toDouble(fencePre.split(",")[1]), Convert.toDouble(fencePre.split(",")[0]));
                    GlobalCoordinates target = new GlobalCoordinates(Convert.toDouble(fenceAfter.split(",")[1]), Convert.toDouble(fenceAfter.split(",")[0]));
                    // 创建GeodeticCalculator,调用计算方法
                    double ellipsoidalDistance = new GeodeticCalculator().calculateGeodeticCurve(Ellipsoid.WGS84, source, target).getEllipsoidalDistance();
                    distance = NumberUtil.add(distance, ellipsoidalDistance);
                }
            }
            // 组装数据
            areaCheckPercent.setAreaId(key);
            areaCheckPercent.setTotalDistance(distance);
            double percent;
            if (value.size() == 0) {
                percent = 100.00;
            } else {
                percent = NumberUtil.mul(NumberUtil.div(finish.size(), value.size(), 4), 100);
            }
            areaCheckPercent.setPercent(percent);
            result.add(areaCheckPercent);
        }
        // 对result中补齐缺失的镇以及对镇名称赋值
        TlArea tlArea = new TlArea();
        tlArea.setAreaType("1");
        List<TlArea> areas = tlAreaService.selectTlAreaList(tlArea);
        Map<Long, List<AreaCheckPercent>> collect = result.stream().collect(Collectors.groupingBy(AreaCheckPercent::getAreaId));
        List<AreaCheckPercent> areaStatics = new ArrayList<>();
        for (TlArea area : areas) {
            AreaCheckPercent areaCheckPercent = new AreaCheckPercent();
            areaCheckPercent.setAreaId(area.getId());
            areaCheckPercent.setAreaName(area.getAreaName());
            List<AreaCheckPercent> areaCheckPercents = collect.get(area.getId());
            if (areaCheckPercents != null && areaCheckPercents.size() > 0) {
                areaCheckPercent.setTotalDistance(areaCheckPercents.get(0).getTotalDistance());
                areaCheckPercent.setPercent(areaCheckPercents.get(0).getPercent());
            } else {
                areaCheckPercent.setTotalDistance(0);
                areaCheckPercent.setPercent(100.00);
            }
            areaStatics.add(areaCheckPercent);
        }
        /*if (collect.containsKey(null)) {
            // 将未知的数据也并入list中
            AreaCheckPercent areaCheckPercents = collect.get(null).get(0);
            areaCheckPercents.setAreaName("未知");
            areaStatics.add(areaCheckPercents);
        }*/
        // 对areaStatics做百分比排序
        areaStatics = areaStatics.stream().sorted(Comparator.comparing(AreaCheckPercent::getPercent).reversed()).collect(Collectors.toList());
        mapResult.put("areaStatics", areaStatics);
        mapResult.put("totalStatic", totalCount == 0 ? 0.00 : NumberUtil.mul(NumberUtil.div(finishCount, totalCount, 4), 100));
        return mapResult;
    }

    /**
     * 根据时间区间查询巡检打卡统计情况
     *
     * @param startDate
     * @param endDate
     * @return
     */
    @Override
    public List<AreaCheckInfoStatic> queryCheckInfo(String startDate, String endDate) {
        List<TlPlanRecord> planRecords = tlPlanRecordMapper.selectTlPlanRecordListByDatePeriod(null, startDate, endDate);
        // 以各个镇分组
        Map<Long, List<TlPlanRecord>> areaCollect = planRecords.stream().filter(a -> ObjectUtil.isNotNull(a.getAreaId())).collect(Collectors.groupingBy(TlPlanRecord::getAreaId));
        List<AreaCheckInfoStatic> list = new ArrayList<>();
        for (Map.Entry<Long, List<TlPlanRecord>> longListEntry : areaCollect.entrySet()) {
            AreaCheckInfoStatic areaCheckInfoStatic = new AreaCheckInfoStatic();
            Long key = longListEntry.getKey();
            List<TlPlanRecord> value = longListEntry.getValue();
            // 对每一个镇的完成数据筛选出来
            List<TlPlanRecord> finish = value.stream().filter(a -> StrUtil.isNotBlank(a.getDeviceCode())).collect(Collectors.toList());
            // 计算时间区间一共几天
            long betweenDay = DateUtil.betweenDay(DateUtil.parseDate(startDate), DateUtil.offsetDay(DateUtil.parseDate(endDate), 1), false);

            areaCheckInfoStatic.setAreaId(key);
            areaCheckInfoStatic.setLocationPoint(value.size());
            areaCheckInfoStatic.setCheckPoint(finish.size());
            areaCheckInfoStatic.setDailyPoint(NumberUtil.div(Convert.toStr(finish.size()), Convert.toStr(betweenDay), 1));
            areaCheckInfoStatic.setStandardPercent(value.size() == 0 ? 0 : NumberUtil.mul(NumberUtil.div(finish.size(), value.size(), 4), 100));
            areaCheckInfoStatic.setUnStandardPercent(value.size() == 0 ? 0 : NumberUtil.sub(100, areaCheckInfoStatic.getStandardPercent()));
            list.add(areaCheckInfoStatic);
        }
        // 对result中补齐缺失的镇以及对镇名称赋值
        TlArea tlArea = new TlArea();
        tlArea.setAreaType("1");
        List<TlArea> areas = tlAreaService.selectTlAreaList(tlArea);
        Map<Long, List<AreaCheckInfoStatic>> collect = list.stream().collect(Collectors.groupingBy(AreaCheckInfoStatic::getAreaId));
        List<AreaCheckInfoStatic> areaStatics = new ArrayList<>();
        for (TlArea area : areas) {
            AreaCheckInfoStatic areaCheckInfoStatic = new AreaCheckInfoStatic();
            areaCheckInfoStatic.setAreaId(area.getId());
            areaCheckInfoStatic.setAreaName(area.getAreaName());
            List<AreaCheckInfoStatic> areaCheckPercents = collect.get(area.getId());
            if (areaCheckPercents != null && areaCheckPercents.size() > 0) {
                areaCheckInfoStatic.setLocationPoint(areaCheckPercents.get(0).getLocationPoint());
                areaCheckInfoStatic.setCheckPoint(areaCheckPercents.get(0).getCheckPoint());
                areaCheckInfoStatic.setDailyPoint(areaCheckPercents.get(0).getDailyPoint());
                areaCheckInfoStatic.setStandardPercent(areaCheckPercents.get(0).getStandardPercent());
                areaCheckInfoStatic.setUnStandardPercent(areaCheckPercents.get(0).getUnStandardPercent());
            }
            areaStatics.add(areaCheckInfoStatic);
        }

        return areaStatics;
    }


}
